@@ -4,7 +4,7 @@
  * Project: GGZCards Server
  * Date: 8/4/99
  * Desc: NetSpades algorithms for Spades AI
- * $Id: spades.c 7268 2005-06-10 12:28:19Z josef $
+ * $Id: spades.c 8540 2006-08-26 18:01:03Z oojah $
  *
  * This file contains the AI functions for playing spades.
  * The AI routines were adapted from Britt Yenne's spades game for
@@ -54,7 +54,7 @@
 #define minimum_team_bid 3	/* FIXME */
 
 #define IS_GOING_NIL(p) (bids[p].sbid.spec == SPADES_NIL || \
-			 bids[p].sbid.spec == SPADES_DOUBLE_NIL)
+			 bids[p].sbid.spec == SPADES_DNIL)
 
 /* #define USE_AI_TRICKS */
 
@@ -680,7 +680,20 @@ bid_t get_bid(bid_t * bid_choices, int bid_count)
 	bid_t bid;
 	char suit;
 
-	bid.bid = 0;		/* reset bid */
+	/* The ai will never bid blind (double) nil. */
+	if (bid_count == 2) {
+		if (bid_choices[0].sbid.spec == SPADES_NO_BLIND) {
+			return bid_choices[0];
+		} else if (bid_choices[1].sbid.spec == SPADES_NO_BLIND) {
+			return bid_choices[1];
+		}
+	}
+
+	/* reset bid */
+	bid.bid = 0;
+	bid.sbid.val = 0;
+	bid.sbid.suit = NO_SUIT;
+	bid.sbid.spec = SPADES_BID;
 
 	/* Count winners in all suits. */
 	points = count_spade_tricks();
@@ -700,6 +713,7 @@ bid_t get_bid(bid_t * bid_choices, int bid_count)
 	}
 
 	bid.sbid.val = find_final_bid(points);
+
 	ggz_debug(DBG_BID, "Final bid: %d", bid.sbid.val);
 	return bid;
 }
@@ -861,7 +875,9 @@ card_t get_play(int play_seat, bool *valid_plays)
 static void Calculate(struct play *play, int agg)
 {
 	int mask, map, count, danger, trump, n, cover, sCount, o;
-	card_t high_card = ggzcards.players[high].table_card;
+	card_t high_card =
+		(high ==
+		 -1) ? UNKNOWN_CARD : ggzcards.players[high].table_card;
 	char suit_lead = ggzcards.players[get_leader()].table_card.suit;
 	char s, r;
 	hand_t *hand = &ggzcards.players[ME].hand;
@@ -1134,7 +1150,9 @@ static int CoverNil(int agg)
 	int map[4];
 	char suit_lead = ggzcards.players[get_leader()].table_card.suit;
 	card_t pard_card = ggzcards.players[PARTNER].table_card;
-	card_t high_card = ggzcards.players[high].table_card;
+	card_t high_card =
+		(high ==
+		 -1) ? UNKNOWN_CARD : ggzcards.players[high].table_card;
 
 	ggz_debug(DBG_PLAY, "Strategy: cover nil");
 
