@@ -4,7 +4,7 @@
  * Project: GGZCards Server
  * Date: 07/03/2001
  * Desc: Game-dependent game functions for Hearts
- * $Id: hearts.c 4400 2002-09-03 17:26:05Z jdorje $
+ * $Id: hearts.c 8561 2006-08-31 08:00:24Z jdorje $
  *
  * Copyright (C) 2001-2002 Brent Hendricks.
  *
@@ -24,7 +24,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#  include <config.h>			/* Site-specific config */
+#  include <config.h>	/* Site-specific config */
 #endif
 
 #include <stdio.h>
@@ -35,6 +35,7 @@
 #include "message.h"
 #include "net.h"
 #include "options.h"
+#include "score.h"
 
 #include "hearts.h"
 
@@ -81,6 +82,7 @@ game_data_t hearts_data = {
 	hearts_handle_option,
 	hearts_get_option_text,
 	game_set_player_message,
+	game_sync_player,
 	game_get_bid_text,
 	game_get_bid_desc,
 	hearts_start_bidding,
@@ -107,7 +109,7 @@ game_data_t hearts_data = {
 static bool hearts_is_valid_game(void)
 {
 	return game.num_players >= MIN_HEARTS_PLAYERS
-	       && game.num_players <= MAX_HEARTS_PLAYERS;
+	    && game.num_players <= MAX_HEARTS_PLAYERS;
 }
 
 static void hearts_init_game(void)
@@ -119,30 +121,29 @@ static void hearts_init_game(void)
 	for (s = 0; s < game.num_seats; s++)
 		assign_seat(s, s);	/* one player per seat */
 
-	game.trump = -1;	/* no trump in hearts */
+	set_trump_suit(NO_SUIT);
 	game.target_score = 100;
 }
 
 static void hearts_get_options(void)
 {
-	add_option("jack_diamonds",
-	           "The jack of diamonds rule is that whoever wins the jack "
-	           "of diamonds gets -10 points.",
-	           1, FALSE, "Use the jack of diamonds rule");
-	add_option("no_blood",
-	           "If selected, nobody will be allowed to play point cards "
-	           "on the first trick.",
-	           1, TRUE, "No blood on the first trick");
-	add_option("num_decks",
-	           "How many decks should be used?",
-	           1, FALSE, "Play with two decks");
-	add_option("target_score",
-	           "How many points until the game is over?",
-	           5,
-	           2,
-	           "Game to 20", "Game to 50",
-		   "Game to 100", "Game to 1000",
-	           "Unending game");
+	add_option("Hearts Options", "jack_diamonds",
+		   "The jack of diamonds rule is that whoever wins the jack "
+		   "of diamonds gets -10 points.",
+		   1, FALSE, "Use the jack of diamonds rule");
+	add_option("Hearts Options", "no_blood",
+		   "If selected, nobody will be allowed to play point cards "
+		   "on the first trick.",
+		   1, TRUE, "No blood on the first trick");
+	add_option("Hearts Options", "num_decks",
+		   "How many decks should be used?",
+		   1, FALSE, "Play with two decks");
+	add_option("Hearts Options", "target_score",
+		   "How many points until the game is over?",
+		   5,
+		   2,
+		   "Game to 20", "Game to 50",
+		   "Game to 100", "Game to 1000", "Unending game");
 	game_get_options();
 }
 
@@ -158,22 +159,22 @@ static int hearts_handle_option(char *option, int value)
 		seat_t s;
 		GHEARTS.num_decks = value + 1;
 		game.deck_type =
-			(value == 0) ? GGZ_DECK_FULL : GGZ_DECK_DOUBLE;
+		    (value == 0) ? GGZ_DECK_FULL : GGZ_DECK_DOUBLE;
 		destroy_deck(game.deck);
 		game.deck = create_deck(game.deck_type);
 
 		/* allocate hands */
 		game.max_hand_length =
-			get_deck_size(game.deck) / game.num_players;
+		    get_deck_size(game.deck) / game.num_players;
 		for (s = 0; s < game.num_seats; s++) {
 			if (game.seats[s].hand.cards)
 				ggz_free(game.seats[s].hand.cards);
 			game.seats[s].hand.cards =
-				ggz_malloc(game.max_hand_length *
-					   sizeof(card_t));
+			    ggz_malloc(game.max_hand_length *
+				       sizeof(card_t));
 		}
 	} else if (!strcmp("target_score", option)) {
-   		switch (value) {
+		switch (value) {
 		case 0:
 			game.target_score = 20;
 			break;
@@ -213,9 +214,10 @@ static char *hearts_get_option_text(char *buf, int bufsz, char *option,
 				 value + 1);
 	} else if (!strcmp(option, "target_score")) {
 		if (game.target_score == MAX_TARGET_SCORE)
-			snprintf(buf, bufsz, "The game will never end."); 	
+			snprintf(buf, bufsz, "The game will never end.");
 		else
-			snprintf(buf, bufsz, "The game is being played to %d.",
+			snprintf(buf, bufsz,
+				 "The game is being played to %d.",
 				 game.target_score);
 	} else
 		return game_get_option_text(buf, bufsz, option, value);
@@ -224,15 +226,16 @@ static char *hearts_get_option_text(char *buf, int bufsz, char *option,
 
 static bool hearts_test_for_gameover(void)
 {
-	player_t p;
+	team_t t;
 	int num_winners = 0, low_score = -1;
 
 	/* We play until the highest-scoring person reaches the target score,
 	   but we can never end in a tie. */
 	if (!game_test_for_gameover())
 		return FALSE;
-	for (p = 0; p < game.num_players; p++) {
-		int score = game.players[p].score;
+	for (t = 0; t < game.num_teams; t++) {
+		int score = get_team_score(t);
+
 		if (low_score < 0 || score < low_score) {
 			low_score = score;
 			num_winners = 1;
@@ -254,18 +257,21 @@ static void hearts_handle_gameover(void)
 	   can be higher than the "target score", since we can never
 	   end in a tie (we just play another hand). */
 	for (p = 0; p < game.num_players; p++) {
-		int score = game.players[p].score;
+		team_t t = game.players[p].team;
+		int score = get_team_score(t);
+
+		assert(t >= 0 && t < game.num_teams);
 		max_score = MAX(score, max_score);
 		if (low_score < 0 || score < low_score) {
 			num_winners = 1;
 			winners[0] = p;
-			low_score = game.players[p].score;
+			low_score = score;
 		} else if (score == low_score) {
 			winners[num_winners] = p;
 			num_winners++;
 		}
 	}
-	
+
 	assert(num_winners == 1);
 	assert(max_score >= game.target_score);
 	handle_gameover_event(num_winners, winners);
@@ -302,8 +308,7 @@ static void hearts_start_playing(void)
 			/* TODO: this only works because the cards are sorted 
 			   this way */
 			card_t card =
-				game.seats[game.players[p].seat].hand.
-				cards[0];
+			    game.seats[game.players[p].seat].hand.cards[0];
 			if (card.suit == CLUBS && card.face == face)
 				game.leader = p;
 		}
@@ -357,8 +362,7 @@ static char *hearts_verify_play(player_t p, card_t card)
 		card_t c;
 
 		/* Check to make sure they have a non-point card. */
-		for (i = 0; i < game.seats[s].hand.hand_size;
-		     i++) {
+		for (i = 0; i < game.seats[s].hand.hand_size; i++) {
 			c = game.seats[s].hand.cards[i];
 			if (c.suit == HEARTS)
 				continue;
@@ -417,7 +421,8 @@ static void hearts_end_hand(void)
 
 		if (GHEARTS.jack_winner == p)
 			fullscore -= 10;
-		game.players[p].score += fullscore;
+		assert(p == game.players[p].team);
+		change_score(p, fullscore);
 
 		if (score == -26) {
 			snprintf(buf, sizeof(buf), "%s shot the moon.",
