@@ -4,7 +4,7 @@
  * Project: GGZCards Server
  * Date: 07/03/2001
  * Desc: Game-dependent game functions for La Pocha
- * $Id: lapocha.c 7433 2005-08-15 09:55:03Z josef $
+ * $Id: lapocha.c 8880 2007-01-09 17:22:14Z josef $
  *
  * Copyright (C) 2001-2002 Brent Hendricks.
  *
@@ -24,13 +24,14 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#  include <config.h>			/* Site-specific config */
+#  include <config.h>	/* Site-specific config */
 #endif
 
 #include <stdio.h>
 
 #include <ggz.h>
 
+#include "ggz_dio.h"
 #include "net_common.h"
 
 #include "bid.h"
@@ -38,6 +39,7 @@
 #include "game.h"
 #include "message.h"
 #include "net.h"
+#include "score.h"
 
 #include "lapocha.h"
 
@@ -61,8 +63,8 @@ static void lapocha_set_player_message(player_t p);
 static void lapocha_end_hand(void);
 
 /* these send lapocha-specific game messages. */
-static int lap_send_trump_request(player_t p);
-static int lap_send_bid_request(player_t p);
+static void lap_send_trump_request(player_t p);
+static void lap_send_bid_request(player_t p);
 static void lap_send_dealer(void);
 static void lap_send_trump(void);
 static void lap_send_bid(player_t bidder, bid_t bid);
@@ -71,13 +73,14 @@ static void lap_send_scores(void);
 game_data_t lapocha_data = {
 	"lapocha",
 	N_("La Pocha"),
-	"http://www.ggzgamingzone.org/games/lapocha/",
+	"http://www.ggzgamingzone.org/gameservers/ggzcards/",
 	lapocha_is_valid_game,
 	lapocha_init_game,
 	game_get_options,
 	game_handle_option,
 	game_get_option_text,
 	lapocha_set_player_message,
+	game_sync_player,
 	lapocha_get_bid_text,
 	game_get_bid_desc,
 	lapocha_start_bidding,
@@ -118,7 +121,7 @@ static void lapocha_init_game(void)
 	game.deck_type = GGZ_DECK_LAPOCHA;
 	game.max_hand_length = 10;
 	game.must_overtrump = 1;	/* in La Pocha, you *must* overtrump
-					   if you can */	
+					   if you can */
 }
 
 static void lapocha_handle_gameover(void)
@@ -130,7 +133,7 @@ static void lapocha_handle_gameover(void)
 
 	/* in La Pocha, anyone who has 0 at the end automatically wins */
 	for (p = 0; p < game.num_players; p++) {
-		if (game.players[p].score == 0) {
+		if (get_player_score(p) == 0) {
 			winners[winner_cnt] = p;
 			winner_cnt++;
 		}
@@ -140,11 +143,13 @@ static void lapocha_handle_gameover(void)
 		/* in the default case, just take the highest score(s) this
 		   should automatically handle the case of teams! */
 		for (p = 0; p < game.num_players; p++) {
-			if (game.players[p].score > hi_score) {
+			int score = get_player_score(p);
+
+			if (score > hi_score) {
 				winner_cnt = 1;
 				winners[0] = p;
-				hi_score = game.players[p].score;
-			} else if (game.players[p].score == hi_score) {
+				hi_score = score;
+			} else if (score == hi_score) {
 				winners[winner_cnt] = p;
 				winner_cnt++;
 			}
@@ -156,9 +161,8 @@ static void lapocha_handle_gameover(void)
 
 static void set_trump(char suit)
 {
-	game.trump = suit;
-	set_global_message("", "Trump is %s.",
-	                   get_suit_name(game.trump));
+	set_trump_suit(suit);
+	set_global_message("", "Trump is %s.", get_suit_name(game.trump));
 	lap_send_trump();
 }
 
@@ -182,20 +186,21 @@ static void lapocha_get_bid(void)
 		char suit;
 		for (suit = 0; suit < 4; suit++)
 			add_sbid(0, suit, LAPOCHA_TRUMP);
-		(void) lap_send_trump_request(game.dealer);
+		(void)lap_send_trump_request(game.dealer);
 		request_client_bid(game.dealer);
-	} else {		/* get a player's numerical bid */
+	} else {	/* get a player's numerical bid */
 		int i;
-		assert(game.next_bid == (game.dealer + game.bid_count) % 4);
+		assert(game.next_bid ==
+		       (game.dealer + game.bid_count) % 4);
 		for (i = 0; i <= game.hand_size; i++) {
 			/* the dealer can't make the sum of the bids equal
 			   the hand size; somebody's got to go down */
 			if (game.next_bid == game.dealer &&
 			    LAPOCHA.bid_sum + i == game.hand_size)
 				continue;
-			add_sbid(i, 0, 0);
+			add_sbid(i, NO_SUIT, LAPOCHA_BID);
 		}
-		(void) lap_send_bid_request(game.next_bid);
+		(void)lap_send_bid_request(game.next_bid);
 		request_client_bid(game.next_bid);
 	}
 }
@@ -239,13 +244,14 @@ static void lapocha_deal_hand(void)
 	/* in la pocha, there are a predetermined number of cards each hand,
 	   as defined by lap_card_count above */
 	game.hand_size = lap_card_count[game.hand_num];
-	game.trump = -1;	/* must be determined later *//* TODO: shouldn't go here */
 
+	/* must be determined later */
+	/* TODO: shouldn't go here */
+	set_trump_suit(NO_SUIT);
 
 	/* in a regular deal, we just deal out hand_size cards to everyone */
 	for (s = 0; s < game.num_seats; s++)
-		deal_hand(game.deck, game.hand_size,
-				&game.seats[s].hand);
+		deal_hand(game.deck, game.hand_size, &game.seats[s].hand);
 }
 
 static int lapocha_get_bid_text(char *buf, size_t buf_len, bid_t bid)
@@ -253,7 +259,7 @@ static int lapocha_get_bid_text(char *buf, size_t buf_len, bid_t bid)
 	if (bid.sbid.spec == LAPOCHA_TRUMP)
 		return snprintf(buf, buf_len, "%s",
 				get_suit_name(bid.sbid.suit % 4));
-	return snprintf(buf, buf_len, "%d", (int) bid.sbid.val);
+	return snprintf(buf, buf_len, "%d", (int)bid.sbid.val);
 }
 
 static void lapocha_set_player_message(player_t p)
@@ -268,7 +274,7 @@ static void lapocha_set_player_message(player_t p)
 	if (game.state >= STATE_FIRST_TRICK
 	    && game.state <= STATE_WAIT_FOR_PLAY) {
 		add_player_message(s, "Contract: %d\n",
-				   (int) game.players[p].bid.bid);
+				   (int)game.players[p].bid.bid);
 	}
 	add_player_tricks_message(p);
 	add_player_bid_message(p);
@@ -281,14 +287,15 @@ static void lapocha_end_hand(void)
 
 	for (p = 0; p < game.num_players; p++) {
 		ggz_debug(DBG_GAME,
-			    "Player %d/%s got %d tricks on a bid of %d", p,
-			    get_player_name(p), game.players[p].tricks,
-			    (int) game.players[p].bid.bid);
-		if (game.players[p].tricks == game.players[p].bid.bid)
-			game.players[p].score +=
-				10 + (5 * game.players[p].bid.bid);
-		else
-			game.players[p].score -= 5 * game.players[p].bid.bid;
+			  "Player %d/%s got %d tricks on a bid of %d", p,
+			  get_player_name(p), game.players[p].tricks,
+			  (int)game.players[p].bid.bid);
+		assert(p == game.players[p].team);
+		if (game.players[p].tricks == game.players[p].bid.bid) {
+			change_score(p, 10 + 5 * game.players[p].bid.bid);
+		} else {
+			change_score(p, -5 * game.players[p].bid.bid);
+		}
 	}
 	lap_send_scores();
 	set_global_message("", "No trump set.");	/* TODO: give
@@ -301,56 +308,53 @@ static void lapocha_end_hand(void)
    games. */
 
 /* these send lapocha-specific game messages. */
-static int lap_send_trump_request(player_t p)
+static void lap_send_trump_request(player_t p)
 {
-	int fd = get_player_socket(p);
-	if (write_opcode(fd, MESSAGE_GAME) < 0 ||
-	    write_opcode(fd, GAME_MESSAGE_GAME) < 0 ||
-	    ggz_write_string(fd, "lapocha") < 0 ||
-	    ggz_write_int(fd, 1) < 0 || ggz_write_char(fd, LAP_REQ_TRUMP) < 0)
-		return -1;
-	return 0;
+	GGZDataIO *dio = get_player_dio(p);
+
+	ggz_dio_packet_start(dio);
+	write_opcode(dio, MSG_GAME_SPECIFIC);
+	ggz_dio_put_char(dio, LAP_REQ_TRUMP);
+	ggz_dio_packet_end(dio);
 }
 
-static int lap_send_bid_request(player_t p)
+static void lap_send_bid_request(player_t p)
 {
-	int fd = get_player_socket(p);
-	if (write_opcode(fd, MESSAGE_GAME) < 0 ||
-	    write_opcode(fd, GAME_MESSAGE_GAME) < 0 ||
-	    ggz_write_string(fd, "lapocha") < 0 ||
-	    ggz_write_int(fd, 1) < 0 || ggz_write_char(fd, LAP_REQ_BID) < 0)
-		return -1;
-	return 0;
+	GGZDataIO *dio = get_player_dio(p);
+
+	ggz_dio_packet_start(dio);
+	write_opcode(dio, MSG_GAME_SPECIFIC);
+	ggz_dio_put_char(dio, LAP_REQ_BID);
+	ggz_dio_packet_end(dio);
 }
 
 static void lap_send_dealer(void)
 {
 	int p;
-	
-	for (p = 0; p < game.num_players; p++) {
-		int fd = get_player_socket(p);
 
-		write_opcode(fd, MESSAGE_GAME);
-		write_opcode(fd, GAME_MESSAGE_GAME);
-		ggz_write_string(fd, "lapocha");
-		ggz_write_int(fd, 5);
-		ggz_write_char(fd, LAP_MSG_DEALER);
-		ggz_write_int(fd, CONVERT_SEAT(game.dealer, p));
+	for (p = 0; p < game.num_players; p++) {
+		GGZDataIO *dio = get_player_dio(p);
+
+		ggz_dio_packet_start(dio);
+		write_opcode(dio, MSG_GAME_SPECIFIC);
+		ggz_dio_put_char(dio, LAP_MSG_DEALER);
+		ggz_dio_put_int(dio, CONVERT_SEAT(game.dealer, p));
+		ggz_dio_packet_end(dio);
 	}
 }
 
 static void lap_send_trump(void)
 {
 	int p;
-	
+
 	for (p = 0; p < game.num_players; p++) {
-		int fd = get_player_socket(p);
-		write_opcode(fd, MESSAGE_GAME);
-		write_opcode(fd, GAME_MESSAGE_GAME);
-		ggz_write_string(fd, "lapocha");
-		ggz_write_int(fd, 2);
-		ggz_write_char(fd, LAP_MSG_TRUMP);
-		ggz_write_char(fd, game.trump);
+		GGZDataIO *dio = get_player_dio(p);
+
+		ggz_dio_packet_start(dio);
+		write_opcode(dio, MSG_GAME_SPECIFIC);
+		ggz_dio_put_char(dio, LAP_MSG_TRUMP);
+		ggz_dio_put_char(dio, game.trump);
+		ggz_dio_packet_end(dio);
 	}
 }
 
@@ -358,18 +362,18 @@ static void lap_send_bid(player_t bidder, bid_t bid)
 {
 	int p;
 	int the_bid = bid.sbid.val;
-	
+
 	assert(game.players[bidder].seat == bidder);
-	
+
 	for (p = 0; p < game.num_players; p++) {
-		int fd = get_player_socket(p);
-		write_opcode(fd, MESSAGE_GAME);
-		write_opcode(fd, GAME_MESSAGE_GAME);
-		ggz_write_string(fd, "lapocha");
-		ggz_write_int(fd, 9);
-		ggz_write_char(fd, LAP_MSG_BID);
-		ggz_write_int(fd, bidder);
-		ggz_write_int(fd, the_bid);
+		GGZDataIO *dio = get_player_dio(p);
+
+		ggz_dio_packet_start(dio);
+		write_opcode(dio, MSG_GAME_SPECIFIC);
+		ggz_dio_put_char(dio, LAP_MSG_BID);
+		ggz_dio_put_int(dio, bidder);
+		ggz_dio_put_int(dio, the_bid);
+		ggz_dio_packet_end(dio);
 	}
 }
 
@@ -378,17 +382,19 @@ static void lap_send_scores(void)
 	player_t p;
 
 	for (p = 0; p < game.num_players; p++) {
-		int fd = get_player_socket(p);
+		GGZDataIO *dio = get_player_dio(p);
 		seat_t s_r;
-		write_opcode(fd, MESSAGE_GAME);
-		write_opcode(fd, GAME_MESSAGE_GAME);
-		ggz_write_string(fd, "lapocha");
-		ggz_write_int(fd, 17);
-		ggz_write_char(fd, LAP_MSG_SCORES);
+
+		ggz_dio_packet_start(dio);
+		write_opcode(dio, MSG_GAME_SPECIFIC);
+		ggz_dio_put_char(dio, LAP_MSG_SCORES);
 		for (s_r = 0; s_r < game.num_seats; s_r++) {
 			seat_t s_abs = UNCONVERT_SEAT(s_r, p);
+
 			assert(game.seats[s_abs].player == s_abs);
-			ggz_write_int(fd, game.players[s_abs].score);
+			assert(game.players[s_abs].team == s_abs);
+			ggz_dio_put_int(dio, get_team_score(s_abs));
 		}
+		ggz_dio_packet_end(dio);
 	}
 }
