@@ -4,7 +4,7 @@
  * Project: GGZCards Server
  * Date: 07/03/2001
  * Desc: Game-dependent game functions for Sueca
- * $Id: sueca.c 4114 2002-04-29 18:28:35Z jdorje $
+ * $Id: sueca.c 8561 2006-08-31 08:00:24Z jdorje $
  *
  * Copyright (C) 2001-2002 Ismael Orenstein
  *
@@ -24,7 +24,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#  include <config.h>			/* Site-specific config */
+#  include <config.h>	/* Site-specific config */
 #endif
 
 #include <stdio.h>
@@ -33,6 +33,7 @@
 #include "common.h"
 #include "game.h"
 #include "message.h"
+#include "score.h"
 #include "team.h"
 
 #include "sueca.h"
@@ -62,6 +63,7 @@ game_data_t sueca_data = {
 	game_handle_option,
 	game_get_option_text,
 	sueca_set_player_message,
+	game_sync_player,
 	game_get_bid_text,
 	game_get_bid_desc,
 	sueca_start_bidding,
@@ -95,7 +97,7 @@ static void sueca_init_game(void)
 	seat_t s;
 
 	game.specific = ggz_malloc(sizeof(sueca_game_t));
-	
+
 	set_num_seats(game.num_players);
 	set_num_teams(2);
 	for (s = 0; s < game.num_seats; s++) {
@@ -107,7 +109,7 @@ static void sueca_init_game(void)
 	game.target_score = 4;
 	/* Must break trump */
 	game.must_break_trump = 1;
-	
+
 	/* Use the SUECA deck */
 	game.deck_type = GGZ_DECK_SUECA;
 	/* Point to the game rules */
@@ -192,26 +194,22 @@ static void sueca_end_hand(void)
 	points = GSUECA.points_on_hand[0] + GSUECA.points_on_hand[2];
 	/* TODO: Must add "dar uma bandeira" */
 	if (points > 90) {
-		game.players[0].score += 2;
-		game.players[2].score += 2;
+		change_score(0, 2);
 		set_global_message("",
 				   "%s/%s got more than 90 points and so got 2 games",
 				   get_player_name(0), get_player_name(2));
 	} else if (points > 60) {
-		game.players[0].score++;
-		game.players[2].score++;
+		change_score(0, 1);
 		set_global_message("",
 				   "%s/%s got more than 60 points and won that hand",
 				   get_player_name(0), get_player_name(2));
 	} else if (points > 30) {
-		game.players[1].score++;
-		game.players[3].score++;
+		change_score(1, 1);
 		set_global_message("",
 				   "%s/%s got more than 60 points and won that hand",
 				   get_player_name(1), get_player_name(3));
 	} else {
-		game.players[1].score += 2;
-		game.players[3].score += 2;
+		change_score(1, 2);
 		set_global_message("",
 				   "%s/%s got more than 90 points and so got 2 games",
 				   get_player_name(1), get_player_name(3));
@@ -226,10 +224,11 @@ static void sueca_deal_hand(void)
 	seat_t s;
 	game.hand_size = 10;
 	for (s = 0; s < game.num_seats; s++)
-		deal_hand(game.deck, game.hand_size,
-				&game.seats[s].hand);
-	game.trump = game.seats[game.dealer].hand.cards[random() % 10].suit;
-	set_global_message("Trump", "Trump is %s.", get_suit_name(game.trump));
+		deal_hand(game.deck, game.hand_size, &game.seats[s].hand);
+	set_trump_suit(game.seats[game.dealer].hand.cards[random() % 10].
+		       suit);
+	set_global_message("Trump", "Trump is %s.",
+			   get_suit_name(game.trump));
 	set_global_message("", "Trump is %s.", get_suit_name(game.trump));
 }
 
@@ -249,11 +248,14 @@ static void sueca_set_player_message(player_t p)
 	   case 1: rubber[0] = 'X'; } */
 
 	score_this_hand =
-		GSUECA.points_on_hand[p] + GSUECA.points_on_hand[(p + 2) % 4];
-	if (!(game.state == STATE_WAIT_FOR_PLAY && game.players[p].is_playing))
+	    GSUECA.points_on_hand[p] + GSUECA.points_on_hand[(p + 2) % 4];
+	if (!
+	    (game.state == STATE_WAIT_FOR_PLAY
+	     && game.players[p].is_playing))
 		sprintf(status, " ");
-	put_player_message(s, "Score:\nIn the game: %d\nIn the hand: %d\n%s",
-			   game.players[p].score, score_this_hand, status);
+	put_player_message(s,
+			   "Score:\nIn the game: %d\nIn the hand: %d\n%s",
+			   get_player_score(p), score_this_hand, status);
 
 	/* Waiting sprintf(message, " Rubber \n %c \n | \n %c--O--%c \n | \n
 	   %c \nThis hand: %d\n%s", rubber[3], rubber[2], rubber[1],
