@@ -4,7 +4,7 @@
  * Project: GGZCards Server
  * Date: 02/27/2002
  * Desc: Functions and data for tracking teams
- * $Id: team.h 3573 2002-03-16 15:19:46Z jdorje $
+ * $Id: team.h 8456 2006-08-02 06:00:35Z jdorje $
  *
  * Copyright (C) 2001-2002 Brent Hendricks.
  *
@@ -61,7 +61,10 @@ void map_func_to_team(team_t t, void (func)(player_t));
 /** @brief Return the number of tricks the team has taken. */
 int get_team_tricks(team_t t);
 
-/** @brief Return the team's score. */
-int get_team_score(team_t t);
+/** @brief Return the player's team, or NO_TEAM. */
+team_t get_player_team(player_t p);
+
+/** @brief Return the seat's team, or NO_TEAM. */
+team_t get_seat_team(seat_t s);
 
 #endif /* __TEAM_H__ */
