@@ -5,7 +5,7 @@
  * Project: GGZ Tic-Tac-Toe game module
  * Date: 09/10/00
  * Desc: Game functions
- * $Id: game.c,v 1.32 2003/05/09 22:57:43 dr_maux Exp $
+ * $Id: game.c 7535 2005-09-18 11:27:24Z josef $
  *
  * Copyright (C) 2000 - 2002 Josef Spillner
  *
@@ -53,7 +53,7 @@ static void game_loadmap(const char *file)
 {
 	FILE *f;
 	char buf[128];
-	enum states {state_title, state_author, state_version, state_anywhere, state_knights, state_nowhere, state_map};
+	enum states {state_title, state_author, state_version, state_graphics, state_anywhere, state_knights, state_nowhere, state_map};
 	enum qualities {quality_ok, quality_bad, quality_error};
 	int state, quality, i, j;
 	unsigned int mapx, mapy;
@@ -102,6 +102,10 @@ static void game_loadmap(const char *file)
 					break;
 				case state_version:
 					map.version = strdup(buf + 1);
+					state = state_graphics;
+					break;
+				case state_graphics:
+					map.graphics = strdup(buf + 1);
 					state = state_anywhere;
 					break;
 				case state_knights:
@@ -146,7 +150,7 @@ static void game_loadmap(const char *file)
 	}
 	if(verbose)
 	{
-		printf("Try map: [%s] [%s] [%s]\n", map.title, map.author, map.version);
+		printf("Try map: [%s] [%s] [%s] <%s>\n", map.title, map.author, map.version, map.graphics);
 		printf("State is %i, sizes are %i/%i, %i/%i\n", state, x, y, mapx, mapy);
 		printf("Detected %i different players\n", map.players);
 	}
@@ -154,6 +158,7 @@ static void game_loadmap(const char *file)
 	/* More sanity checks */
 	if(state != state_map) quality = quality_error;
 	if((x != mapx) || (y != mapy)) quality = quality_error;
+	if(strcmp(map.version, HASTINGS_MAP_FORMAT)) quality = quality_error;
 	map.height = y;
 	map.width = x;
 
@@ -165,6 +170,7 @@ static void game_loadmap(const char *file)
 		hastings_maps[mapcount].title = map.title;
 		hastings_maps[mapcount].author = map.author;
 		hastings_maps[mapcount].version = map.version;
+		hastings_maps[mapcount].graphics = map.graphics;
 		hastings_maps[mapcount].height = map.height;
 		hastings_maps[mapcount].width = map.width;
 		for(i = 0; i < map.height; i++)
@@ -216,7 +222,7 @@ void game_init(GGZdMod *ggz)
 
 
 /* Handle event from GGZ server */
-void game_handle_ggz(GGZdMod *ggz, GGZdModEvent event, void *data)
+void game_handle_ggz(GGZdMod *ggz, GGZdModEvent event, const void *data)
 {
 	int player;
 
@@ -250,18 +256,20 @@ void game_handle_ggz(GGZdMod *ggz, GGZdModEvent event, void *data)
 /* Calculate if all seats are full */
 static int seats_full(void)
 {
-	return ggzdmod_count_seats(hastings_game.ggz, GGZ_SEAT_OPEN)
-		+ ggzdmod_count_seats(hastings_game.ggz, GGZ_SEAT_RESERVED) == 0;
+	return ggzdmod_count_seats(hastings_game.ggz, GGZ_SEAT_OPEN) == 0
+	  && ggzdmod_count_seats(hastings_game.ggz, GGZ_SEAT_RESERVED) == 0
+	  && ggzdmod_count_seats(hastings_game.ggz, GGZ_SEAT_ABANDONED) == 0;
 }
 
 
 /* Handle message from player */
-void game_handle_player(GGZdMod *ggz, GGZdModEvent event, void *seat_data)
+void game_handle_player(GGZdMod *ggz, GGZdModEvent event,
+			const void *seat_data)
 {
-	int num;
+	const int *num_ptr = seat_data;
+	int num = *num_ptr;
 	int fd, op;
 
-	num = *(int*)seat_data;
 	fd = ggzdmod_get_seat(hastings_game.ggz, num).fd;
 
 	if (fd < 0 || ggz_read_int(fd, &op) < 0) return;
@@ -272,7 +280,7 @@ void game_handle_player(GGZdMod *ggz, GGZdModEvent event, void *seat_data)
 	{
 		case HASTINGS_SND_MOVE:
 			if(game_handle_move(num) == 0)
-				game_update(HASTINGS_EVENT_MOVE, seat_data);
+				game_update(HASTINGS_EVENT_MOVE, &num);
 			break;
 		case HASTINGS_SND_MAP:
 			if(hastings_game.state == HASTINGS_STATE_WAITFORMAP)
@@ -305,7 +313,8 @@ void game_handle_player(GGZdMod *ggz, GGZdModEvent event, void *seat_data)
 
 
 /* Handle message from player */
-void game_handle_spectator(GGZdMod *ggz, GGZdModEvent event, void *spectator_data)
+void game_handle_spectator(GGZdMod *ggz, GGZdModEvent event,
+			   const void *spectator_data)
 {
 	int num;
 	int fd, op;
@@ -359,6 +368,7 @@ static int game_send_maps(int seat)
 		if((ggz_write_string(fd, hastings_maps[i].title) < 0)
 		|| (ggz_write_string(fd, hastings_maps[i].author) < 0)
 		|| (ggz_write_string(fd, hastings_maps[i].version) < 0)
+		|| (ggz_write_string(fd, hastings_maps[i].graphics) < 0)
 		|| (ggz_write_int(fd, hastings_maps[i].width) < 0)
 		|| (ggz_write_int(fd, hastings_maps[i].height) < 0))
 			return -1;
@@ -466,7 +476,7 @@ int game_send_players(void)
 		{
 			seat2 = ggzdmod_get_seat(hastings_game.ggz, i);
 			if(ggz_write_int(fd, seat2.type) < 0) return -1;
-			if((seat.type != GGZ_SEAT_OPEN) && (seat2.name))
+			if((seat2.type != GGZ_SEAT_OPEN) && (seat2.name))
 				if(ggz_write_string(fd, seat2.name) < 0)
 					return -1;
 		}
