@@ -362,12 +362,12 @@ void ZoneGGZModServer::hook_event(GGZdMod *ggzdmod, GGZdModEvent event,
 		case GGZDMOD_EVENT_SPECTATOR_LEAVE:
 		case GGZDMOD_EVENT_SPECTATOR_SEAT:
 		case GGZDMOD_EVENT_SPECTATOR_DATA:
-		case GGZDMOD_EVENT_LOG:
-		case GGZDMOD_EVENT_GAMEREPORT:
-		case GGZDMOD_EVENT_REQ_BOOT:
-		case GGZDMOD_EVENT_REQ_BOT:
-		case GGZDMOD_EVENT_REQ_OPEN:
-		case GGZDMOD_EVENT_REQ_NUM_SEATS:
+		//case GGZDMOD_EVENT_LOG:
+		//case GGZDMOD_EVENT_GAMEREPORT:
+		//case GGZDMOD_EVENT_REQ_BOOT:
+		//case GGZDMOD_EVENT_REQ_BOT:
+		//case GGZDMOD_EVENT_REQ_OPEN:
+		//case GGZDMOD_EVENT_REQ_NUM_SEATS:
 		case GGZDMOD_EVENT_ERROR:
 			// not supported
 			break;
