@@ -4,7 +4,7 @@
  * Project: GGZ Server
  * Date: 06/11/2000
  * Desc: Front-end functions for handling database manipulation
- * $Id: ggzdb.h 7498 2005-08-27 08:45:46Z josef $
+ * $Id: ggzdb.h 8504 2006-08-08 09:07:31Z josef $
  *
  * Copyright (C) 2000 Brent Hendricks.
  *
@@ -40,7 +40,7 @@
 typedef struct {
 	unsigned int user_id;			/* Numeric UID */
 	char handle[MAX_USER_NAME_LEN+1];	/* Players nickname/handle */
-	char password[33];			/* Players password (probably hashed) */
+	char password[41];			/* Players password (probably hashed) */
 	char name[33];				/* Players real name */
 	char email[33];				/* Players email address */
 	time_t last_login;			/* Last login */
@@ -56,6 +56,7 @@ typedef struct {
 
 /* Connection information */
 typedef struct {
+	const char *type;
 	const char *datadir;
 	const char *host;
 	const char *database;
@@ -118,5 +119,11 @@ GGZDBResult ggzdb_stats_newmatch(const char *game, const char *winner, const cha
 /* Register a savegame entry temporarily */
 GGZDBResult ggzdb_stats_savegame(const char *game, const char *owner, const char *savegame);
 
+/* Escape a string for SQL */
+char *_ggz_sql_escape(const char *str);
+
+/* Unescape a string from SQL */
+/*char *_ggz_sql_unescape(const char *str); */
+
 #endif
 
