@@ -4,7 +4,7 @@
  * Project: GGZ Server
  * Date: 10/11/99
  * Desc: Error functions
- * $Id: err_func.c 7714 2006-01-02 16:53:31Z josef $
+ * $Id: err_func.c 8779 2007-01-02 12:14:04Z josef $
  *
  * Copyright (C) 1999 Brent Hendricks.
  *
@@ -175,6 +175,7 @@ void log_msg(const unsigned log_type, const char *fmt, ...)
 {
 	va_list ap;
 	char msg[4096];
+	bool have_msg = false;
 	
 	va_start(ap, fmt);
 
@@ -182,6 +183,7 @@ void log_msg(const unsigned log_type, const char *fmt, ...)
 		int priority;
 
 		vsnprintf(msg, sizeof(msg), fmt, ap);
+		have_msg = true;
 
 		switch(log_type) {
 			case GGZ_LOG_ALWAYS:
@@ -198,7 +200,7 @@ void log_msg(const unsigned log_type, const char *fmt, ...)
 	}
 
 	if(log_type & log_info.log_types_console) {
-		vsnprintf(msg, sizeof(msg), fmt, ap);
+		if(!have_msg) vsnprintf(msg, sizeof(msg), fmt, ap);
 		printf("[LOG] %s\n", msg);
 	}
 
@@ -225,6 +227,13 @@ int logfile_set_facility(char *facstr)
 }
 
 
+/* Initialize some settings for logging */
+void logfile_preinitialize(void)
+{
+	update_info.update_interval = LOG_UPDATE_INTERVAL;
+}
+
+
 /* Initialize the log files */
 void logfile_initialize(void)
 {
@@ -252,7 +261,6 @@ void logfile_initialize(void)
 	}
 
 	pthread_mutex_init(&update_info.mut, NULL);
-	update_info.update_interval = LOG_UPDATE_INTERVAL;
 	update_info.next_update = time(NULL) + update_info.update_interval;
 	update_info.start_time = time(NULL);
 	update_info.anon_users = 0;
@@ -321,7 +329,7 @@ int log_next_update_sec(void)
 	int max_select_wait;
 
 	if((max_select_wait = update_info.next_update - time(NULL)) < 1)
-		max_select_wait = 1;
+		max_select_wait = 0;
 
 	return max_select_wait;
 }
@@ -332,6 +340,7 @@ void log_generate_update(void)
 {
 	int uptime;
 	int anon, regd, login, logout;
+	int tables, tables_created, tables_closed;
 
 	/* Not in critical section (only our thread uses these) */
 	uptime = time(NULL) - update_info.start_time;
@@ -344,6 +353,9 @@ void log_generate_update(void)
 	regd = update_info.regd_users;
 	login = update_info.num_logins;
 	logout = update_info.num_logouts;
+	tables = update_info.tables;
+	tables_created = update_info.tables_created;
+	tables_closed = update_info.tables_closed;
 
 	/* Clear periodic counters */
 	update_info.num_logins = 0;
@@ -362,11 +374,13 @@ void log_generate_update(void)
 		log_msg(GGZ_LOG_UPDATE, "UPDATE Uptime=%d sec", uptime);
 		log_msg(GGZ_LOG_UPDATE, "UPDATE There are %d anonymous users and %d registered users online", anon, regd);
 		log_msg(GGZ_LOG_UPDATE, "UPDATE Since the last update, %d users have logged in, %d logged out", login, logout);
+		log_msg(GGZ_LOG_UPDATE, "UPDATE There are %d tables open", tables);
+		log_msg(GGZ_LOG_UPDATE, "UPDATE Since the last update, %d tables have been created, %d closed", tables_created, tables_closed);
 		
 	} else
 #endif
-		log_msg(GGZ_LOG_UPDATE, "UPDATE %d %d %d %d %d",
-			uptime, anon, regd, login, logout);
+		log_msg(GGZ_LOG_UPDATE, "UPDATE %d %d %d %d %d %d %d %d",
+			uptime, anon, regd, login, logout, tables, tables_created, tables_closed);
 }
 
 
@@ -406,6 +420,24 @@ void log_logout_regd(void)
 }
 
 
+void log_create_table(void)
+{
+	pthread_mutex_lock(&update_info.mut);
+	update_info.tables_created++;
+	update_info.tables++;
+	pthread_mutex_unlock(&update_info.mut);
+}
+
+
+void log_close_table(void)
+{
+	pthread_mutex_lock(&update_info.mut);
+	update_info.tables_closed++;
+	update_info.tables--;
+	pthread_mutex_unlock(&update_info.mut);
+}
+
+
 void log_update_set_interval(int sec)
 {
 	/* This must be called before threading starts.  We could potentially
