@@ -4,7 +4,7 @@
  * Project: GGZ Server
  * Date: 10/11/99
  * Desc: Error function prototypes
- * $Id: err_func.h 7714 2006-01-02 16:53:31Z josef $
+ * $Id: err_func.h 8779 2007-01-02 12:14:04Z josef $
  *
  * Copyright (C) 1999 Brent Hendricks.
  *
@@ -95,6 +95,7 @@ extern LogInfo log_info;
 int logfile_set_facility(char *);
 
 /* Initialize the log files */
+void logfile_preinitialize(void);
 void logfile_initialize(void);
 
 /* Log a normal message */
@@ -108,3 +109,5 @@ void log_login_regd(void);
 void log_logout_anon(void);
 void log_logout_regd(void);
 void log_update_set_interval(int);
+void log_create_table(void);
+void log_close_table(void);
