@@ -4,7 +4,7 @@
  * Project: GGZ Server
  * Date: 5/9/00
  * Desc: Functions for handling/manipulating GGZ events
- * $Id: event.c 5928 2004-02-15 02:43:16Z jdorje $
+ * $Id: event.c 8439 2006-08-01 10:21:43Z oojah $
  *
  * Copyright (C) 2000 Brent Hendricks.
  *
@@ -100,12 +100,6 @@ GGZReturn event_room_enqueue(int room, GGZEventFunc func,
 	GGZEvent *event;
 	int i;
 
-	/* Check for illegal room # */	
-	if(room < 0) {
-		dbg_msg(GGZ_DBG_LISTS, "event_room_enqueue() called from -1");
-		return GGZ_OK;
-	}
-
 	/* Allocate a new event item */
 	event = ggz_malloc(sizeof(GGZEvent));
 	dbg_msg(GGZ_DBG_LISTS, "Allocated event %p", event);
@@ -117,6 +111,21 @@ GGZReturn event_room_enqueue(int room, GGZEventFunc func,
 	event->free = free;
 	event->handle = func;
 
+	/* Check for illegal room # */	
+	if(room < 0) {
+		event_free(event);
+		dbg_msg(GGZ_DBG_LISTS, "event_room_enqueue() called from -1");
+		return GGZ_OK;
+	}
+
+	/* Check for removed room */
+	if (room_is_removed(room)) {
+		event_free(event);
+		dbg_msg(GGZ_DBG_LISTS,
+			"Skipped event (removed room)");
+		return GGZ_OK;
+	}
+
 	pthread_rwlock_wrlock(&rooms[room].lock);
 
 	/* Check for empty room (event might be last player leaving) */
@@ -124,7 +133,7 @@ GGZReturn event_room_enqueue(int room, GGZEventFunc func,
 		pthread_rwlock_unlock(&rooms[room].lock);
 		event_free(event);
 		dbg_msg(GGZ_DBG_LISTS,
-			"Deallocated event %p (empty room)", event);
+			"Skipped event (empty room)");
 		return GGZ_OK;
 	}
 
@@ -507,7 +516,7 @@ GGZReturn event_table_handle(GGZTable* table)
 }
 
 
-/* Flush queued up room-specific events for table */
+/* Flush queued up table-specific events for table */
 GGZReturn event_table_flush(GGZTable* table)
 {
 	GGZEvent *event, *next;
@@ -542,7 +551,7 @@ GGZReturn event_table_flush(GGZTable* table)
  *
  * Receives:
  * GGZTable *table : pointer to table we are adding event to
- * GGZEvent  *event  : pointer to event we're adding
+ * GGZEvent *event : pointer to event we're adding
  *
  * Note: Should be called with table write lock acquired
  */
