@@ -4,7 +4,7 @@
  * Project: GGZ Server
  * Date: 09/24/01
  * Desc: User database editor for ggzd server
- * $Id: ggzduedit.c 7714 2006-01-02 16:53:31Z josef $
+ * $Id: ggzduedit.c 8749 2006-12-25 02:07:57Z jdorje $
  *
  * Copyright (C) 2001 Brent Hendricks.
  *
@@ -38,10 +38,15 @@
 
 #include "ggzdb.h"
 #include "ggzdb_proto.h"
-#define PERMS_DEFINE_STR_TABLE
+#include "datatypes.h" /* For Options type */
 #include "perms.h"
 
 
+/* Server options */
+/* This is required by the database functions
+ * but isn't actually used here */
+Options opt;
+
 char lb[1024];
 ggzdbPlayerEntry pe;
 
@@ -100,38 +105,38 @@ static void list_players(void)
 
 static void show_all_perms(void)
 {
-	int i;
+	GGZPerm p;
 
 	printf("Available permissions:\n");
-	for(i=0; i<8; i++) {
-		printf(" [%s] (bit %i)\n", perms_str_table[i], i);
+	for (p = 0; p < GGZ_PERM_COUNT; p++) {
+		printf(" [%32s] (bit %i)\n", ggz_perm_to_string(p), p);
 	}
 }
 
 
-static void show_perms(unsigned int perms, int add_spaces)
+static void show_perms(GGZPermset perms, int add_spaces)
 {
-	int i;
-	unsigned int testbit=1;
 	int first=1;
 	int col=1;
+	GGZPerm p;
+
+	for (p = 0; p < GGZ_PERM_COUNT; p++) {
+		if (ggz_perms_is_set(perms, p)) {
+			const char *name = ggz_perm_to_string(p);
 
-	for(i=0; i<32; i++) {
-		if((perms & testbit) == testbit) {
 			if(first) {
-				printf("%s", perms_str_table[i]);
+				printf("%24s", name);
 				first = 0;
 			} else if(col) {
-				printf("      %s\n", perms_str_table[i]);
+				printf("        %24s\n", name);
 				col = 0;
 			} else {
 				if(add_spaces)
 					printf("%*c", add_spaces, ' ');
-				printf("               %s", perms_str_table[i]);
+				printf("               %24s", name);
 				col = 1;
 			}
 		}
-		testbit <<= 1;
 	}
 
 	if(first)
@@ -140,13 +145,15 @@ static void show_perms(unsigned int perms, int add_spaces)
 		printf("\n");
 
 	printf("               -> roles: ");
-	if((perms & PERMS_ADMIN_MASK) == PERMS_ADMIN_MASK)
+	if (ggz_perms_is_admin(perms))
 		printf("[admin]");
-	if((perms & PERMS_DEFAULT_SETTINGS) == PERMS_DEFAULT_SETTINGS)
+	if (ggz_perms_is_host(perms))
+		printf("[host]");
+	if (ggz_perms_is_set(perms, GGZ_PERM_ROOMS_LOGIN))
 		printf("[registered]");
-	if((perms & PERMS_DEFAULT_ANON) == PERMS_DEFAULT_ANON)
+	if (ggz_perms_is_set(perms, GGZ_PERM_NO_STATS))
 		printf("[guest]");
-	if(perms & PERMS_CHAT_BOT)
+	if (ggz_perms_is_set(perms, GGZ_PERM_CHAT_BOT))
 		printf("[chatbot]");
 	printf("\n");
 }
@@ -420,11 +427,13 @@ int main(int argc, char **argv)
 		{"dbname", required_argument, 0, 'D'},
 		{"username", required_argument, 0, 'u'},
 		{"password", optional_argument, 0, 'p'},
+		{"type", optional_argument, 0, 't'},
 		{"help", no_argument, 0, 'h'},
 		{"version", no_argument, 0, 'v'},
 		{0, 0, 0, 0}
 	};
 
+	conn.type = NULL;
 	conn.datadir = NULL;
 	conn.database = NULL;
 	conn.host = NULL;
@@ -434,7 +443,7 @@ int main(int argc, char **argv)
 
 	while(1)
 	{
-		opt = getopt_long(argc, argv, "vhd:H:D:u:p::", options, &optindex);
+		opt = getopt_long(argc, argv, "vhd:H:D:u:t:p::", options, &optindex);
 		if(opt == -1) break;
 		switch(opt)
 		{
@@ -450,6 +459,9 @@ int main(int argc, char **argv)
 			case 'u':
 				conn.username = optarg;
 				break;
+			case 't':
+				conn.type = optarg;
+				break;
 			case 'p':
 				if(!optarg)
 				{
@@ -474,6 +486,7 @@ int main(int argc, char **argv)
 				printf("[-H | --host     <hostname>] Host of the database\n");
 				printf("[-u | --username <username>] Database user\n");
 				printf("[-p | --password=[password]] Database password\n");
+				printf("[-t | --type     <dbtype>  ] Type of database (for DBI only)\n");
 				printf("[-h | --help               ] Display this help\n");
 				printf("[-v | --version            ] Display version number only\n");
 				exit(0);
@@ -494,12 +507,14 @@ int main(int argc, char **argv)
 			conn.username = ggz_conf_read_string(rc, "General", "DatabaseUsername", NULL);
 		if(!conn.password)
 			conn.password = ggz_conf_read_string(rc, "General", "DatabasePassword", NULL);
+		if(!conn.type)
+			conn.type = ggz_conf_read_string(rc, "General", "DatabaseType", NULL);
 		ggz_conf_close(rc);
 	}
 
 	if(!conn.datadir) {
 		if(!conn.database) {
-			conn.datadir = DATADIR;
+			conn.datadir = GGZDDATADIR;
 		}
 	}
 
@@ -513,9 +528,18 @@ int main(int argc, char **argv)
 		}
 	}
 
+	if(!strcmp(DATABASE_TYPE, "dbi")) {
+		if(!conn.type) {
+			fprintf(stderr, "Database type '%s' needs plugin type name\n",
+				DATABASE_TYPE);
+			exit(-1);
+		}
+	}
+
 	if((!strcmp(DATABASE_TYPE, "mysql"))
 	|| (!strcmp(DATABASE_TYPE, "pgsql"))
-	|| (!strcmp(DATABASE_TYPE, "sqlite"))) {
+	|| (!strcmp(DATABASE_TYPE, "sqlite"))
+	|| (!strcmp(DATABASE_TYPE, "dbi"))) {
 		if((!conn.database) || (!conn.host) || (!conn.username) || (!conn.password)) {
 			fprintf(stderr, "Database type '%s' needs database access parameters\n",
 				DATABASE_TYPE);
