@@ -4,7 +4,7 @@
  * Project: GGZ Server
  * Date: 9/22/01
  * Desc: Functions for handling network IO
- * $Id: net.c 7901 2006-03-13 12:35:44Z josef $
+ * $Id: net.c 8765 2006-12-27 11:20:31Z josef $
  * 
  * Code for parsing XML streamed from the server
  *
@@ -138,12 +138,16 @@ static void _net_handle_motd(GGZNetIO *net, GGZXMLElement *element);
 static void _net_handle_pong(GGZNetIO *net, GGZXMLElement *element);
 static void _net_handle_ping(GGZNetIO *net, GGZXMLElement *element);
 static void _net_handle_info(GGZNetIO *net, GGZXMLElement *element);
+static void _net_handle_admin(GGZNetIO *net, GGZXMLElement *element);
+static void _net_handle_adminperm(GGZNetIO *net, GGZXMLElement *element);
+static void _net_handle_reason(GGZNetIO *net, GGZXMLElement *element);
 static void _net_handle_tls_start(GGZNetIO *net, GGZXMLElement *element);
 
 /* Utility functions */
 static int str_to_int(const char *str, int dflt);
 static bool check_playerconn(GGZNetIO *net, const char *type);
-static void _net_dump_data(struct _GGZNetIO *net, char *data, int size);
+static void _net_dump_data(struct _GGZNetIO *net, char *data, int size,
+			   bool outgoing);
 static GGZReturn _net_send_result(GGZNetIO *net, const char *action,
 				  GGZClientReqError code);
 static GGZReturn _net_send_login_normal_status(GGZNetIO *net,
@@ -166,6 +170,8 @@ static GGZReturn _net_send_pong(GGZNetIO *net, const char *id);
 
 static GGZReturn _net_send_room_player_count(GGZNetIO *net,
 					     int index, int player_count);
+static GGZReturn _net_send_room_delete(GGZNetIO *net,
+					     int index);
 static GGZReturn _net_send_line(GGZNetIO *net, char *line, ...)
 				ggz__attribute((format(printf, 2, 3)));
 
@@ -215,13 +221,16 @@ GGZNetIO* net_new(int fd, GGZClient *client)
 	return net;
 }
 
-
+/* This is called to set or close the dump file.  Call it with NULL to
+   disable dumping. */
 GGZReturn net_set_dump_file(GGZNetIO *net, const char* filename)
 {
 	if (!filename) {
 		/* Remove any existing dump file. */
 		if (net->dump_file >= 0 && net->dump_file != STDERR_FILENO)
-			close(net->dump_file);
+			if (close(net->dump_file) < 0) {
+				ggz_error_sys("Could not close dump file: ");
+			}
 		net->dump_file = -1;
 		return GGZ_OK;
 	}
@@ -279,6 +288,8 @@ void net_free(GGZNetIO *net)
 		
 		if (net->parser)
 			XML_ParserFree(net->parser);
+
+		net_set_dump_file(net, NULL);
 		
 		ggz_free(net);
 	}
@@ -525,10 +536,11 @@ GGZReturn net_send_player(GGZNetIO *net, GGZPlayer *player)
 
 	/* The caller should ensure that these values are safe to access... */
 	ret = _net_send_line(net, 
-			      "<PLAYER ID='%s' TYPE='%s' TABLE='%d' "
-			      "LAG='%d'%s/>",
-			      player_name_quoted, type_desc, player->table,
-			      player->lag_class, stats);
+			     "<PLAYER ID='%s' TYPE='%s' TABLE='%d' "
+			     "PERMS='0x%08X' LAG='%d'%s/>",
+			     player_name_quoted, type_desc, player->table,
+			     player->perms,
+			     player->lag_class, stats);
 
 	ggz_free(player_name_quoted);
 
@@ -554,6 +566,26 @@ static GGZReturn _net_send_player_lag(GGZNetIO *net, GGZPlayer *player)
 }
 
 
+static GGZReturn _net_send_player_perms(GGZNetIO *net, GGZPlayer *player)
+{
+	GGZPlayerType type = player_get_type(player);
+	const char *type_desc = ggz_playertype_to_string(type);
+	char *player_name_quoted;
+	GGZReturn ret;
+
+	player_name_quoted = ggz_xml_escape(player->name);
+
+	/* The caller should ensure that these values are safe to access... */
+	ret = _net_send_line(net, 
+			     "<PLAYER ID='%s' TYPE='%s' PERMS='0x%08X'/>",
+			     player_name_quoted, type_desc, player->perms);
+
+	ggz_free(player_name_quoted);
+
+	return ret;
+}
+
+
 static GGZReturn _net_send_player_stats(GGZNetIO *net, GGZPlayer *player)
 {
 	char stats[512];
@@ -665,9 +697,11 @@ GGZReturn net_send_chat(GGZNetIO *net, GGZChatType type,
 	sender_quoted = ggz_xml_escape(sender);
 
 	if (msg) {
+		char *msg_quoted = ggz_xml_escape(msg);
 		ret = _net_send_line(net, "<CHAT TYPE='%s' FROM='%s'>"
-				      "<![CDATA[%s]]></CHAT>", 
-				      type_str, sender_quoted, msg);
+				      "%s</CHAT>", 
+				      type_str, sender_quoted, msg_quoted);
+		ggz_free(msg_quoted);
 	} else 
 		ret = _net_send_line(net, "<CHAT TYPE='%s' FROM='%s'/>",
 				      type_str, sender_quoted);
@@ -684,6 +718,12 @@ GGZReturn net_send_chat_result(GGZNetIO *net, GGZClientReqError status)
 }
 
 
+GGZReturn net_send_admin_result(GGZNetIO *net, GGZClientReqError status)
+{
+	return _net_send_result(net, "admin", status);
+}
+
+
 GGZReturn net_send_table_launch(GGZNetIO *net, GGZClientReqError status)
 {
 	return _net_send_result(net, "launch", status);
@@ -797,6 +837,19 @@ GGZReturn net_send_player_update(GGZNetIO *net, GGZPlayerUpdateType opcode,
 			       "ROOM='%d'>", room_id);
 		_net_send_player_stats(net, player);
 		return _net_send_line(net, "</UPDATE>");
+	case GGZ_PLAYER_UPDATE_PERMS:
+		player = hash_player_lookup(name);
+		if (!player) {
+			err_msg("Player lookup failed!");
+			return GGZ_OK;
+		}
+		p2 = *player;
+		pthread_rwlock_unlock(&player->lock);
+
+		_net_send_line(net, "<UPDATE TYPE='player' ACTION='perms' "
+			       "ROOM='%d'>", room_id);
+		_net_send_player_perms(net, &p2);
+		return _net_send_line(net, "</UPDATE>");
 	}
 
 	err_msg("net_send_player_update: unknown opcode %d.", opcode);
@@ -884,8 +937,16 @@ GGZReturn _net_send_room_player_count(GGZNetIO *net,
 }
 
 
+GGZReturn _net_send_room_delete(GGZNetIO *net,
+				      int index)
+{
+	return _net_send_line(net, "<ROOM ID='%d'/>",
+			      index);
+}
+
+
 GGZReturn net_send_room_update(GGZNetIO *net, GGZRoomUpdateType opcode,
-			       int index, int player_count)
+			       int index, int player_count, RoomStruct *room)
 {
 	char *action = NULL;
 
@@ -893,6 +954,15 @@ GGZReturn net_send_room_update(GGZNetIO *net, GGZRoomUpdateType opcode,
 	case GGZ_ROOM_UPDATE_PLAYER_COUNT:
 		action = "players";
 		break;
+	case GGZ_ROOM_UPDATE_ADD:
+		action = "add";
+		break;
+	case GGZ_ROOM_UPDATE_DELETE:
+		action = "delete";
+		break;
+	case GGZ_ROOM_UPDATE_CLOSE:
+		action = "close";
+		break;
 	}
 
 	_net_send_line(net, "<UPDATE TYPE='room' ACTION='%s'>", action);
@@ -901,6 +971,16 @@ GGZReturn net_send_room_update(GGZNetIO *net, GGZRoomUpdateType opcode,
 	case GGZ_ROOM_UPDATE_PLAYER_COUNT:
 		_net_send_room_player_count(net, index, player_count);
 		break;
+	case GGZ_ROOM_UPDATE_ADD:
+		/* TODO: always send verbose? */
+		net_send_room(net, index, room, 1);
+		break;
+	case GGZ_ROOM_UPDATE_DELETE:
+		_net_send_room_delete(net, index);
+		break;
+	case GGZ_ROOM_UPDATE_CLOSE:
+		_net_send_room_delete(net, index);
+		break;
 	}
 
 	return _net_send_line(net, "</UPDATE>");
@@ -986,7 +1066,7 @@ GGZPlayerHandlerStatus net_read_data(GGZNetIO *net)
 		return GGZ_REQ_DISCONNECT;
 	}
 
-	_net_dump_data(net, buf, len);
+	_net_dump_data(net, buf, len, false);
 
 	/* If len == 0 then we've reached EOF */
 	done = (len == 0);
@@ -1080,65 +1160,68 @@ static void _net_parse_text(void *data, const char *text, int len)
 }
 
 
-static void _net_dump_data(GGZNetIO *net, char *data, int size)
+static void _net_dump_data(GGZNetIO *net, char *data, int size, bool outgoing)
 {
-	if (net->dump_file > 0)
+	char marker[32];
+
+	if (net->dump_file > 0) {
+		if(outgoing)
+			snprintf(marker, sizeof(marker), "=> %i ", net->fd);
+		else
+			snprintf(marker, sizeof(marker), "%i => ", net->fd);
+
+		write(net->dump_file, marker, strlen(marker));
 		write(net->dump_file, data, size);
+	}
 }
 
 static GGZXMLElement* _net_new_element(const char *tag,
 				       const char * const *attrs)
 {
-	void (*process_func)();
+	void (*process_func)(GGZNetIO *net, GGZXMLElement *element) = NULL;
+	int i;
+	struct {
+		const char *tag;
+		void (*process_func)(GGZNetIO *net, GGZXMLElement *element);
+	} tags[] = {
+#define TAG(t) {#t, _net_handle_ ## t}
+		TAG(session),
+		TAG(login),
+		TAG(channel),
+		TAG(name),
+		TAG(password),
+		TAG(email),
+		TAG(update),
+		TAG(list),
+		TAG(enter),
+		TAG(chat),
+		TAG(admin),
+		TAG(adminperm),
+		TAG(reason),
+		TAG(info),
+		TAG(join),
+		TAG(leave),
+		TAG(reseat),
+		TAG(launch),
+		TAG(table),
+		TAG(seat),
+		TAG(desc),
+		TAG(motd),
+		TAG(pong),
+		TAG(ping),
+		TAG(tls_start)
+#undef TAG
+	};
+	typedef void (*ggzxmlfunc)(void *, GGZXMLElement *);
 
-	if (strcasecmp(tag, "SESSION") == 0)
-		process_func = _net_handle_session;
-	else if (strcasecmp(tag, "LOGIN") == 0)
-		process_func = _net_handle_login;
-	else if (strcasecmp(tag, "CHANNEL") == 0)
-		process_func = _net_handle_channel;
-	else if (strcasecmp(tag, "NAME") == 0)
-		process_func = _net_handle_name;
-	else if (strcasecmp(tag, "PASSWORD") == 0)
-		process_func = _net_handle_password;
-	else if (strcasecmp(tag, "EMAIL") == 0)
-		process_func = _net_handle_email;
-	else if (strcasecmp(tag, "UPDATE") == 0)
-		process_func = _net_handle_update;
-	else if (strcasecmp(tag, "LIST") == 0)
-		process_func = _net_handle_list;
-	else if (strcasecmp(tag, "ENTER") == 0)
-		process_func = _net_handle_enter;
-	else if (strcasecmp(tag, "CHAT") == 0)
-		process_func = _net_handle_chat;
-	else if (strcasecmp(tag, "INFO") == 0)
-		process_func = _net_handle_info;
-	else if (strcasecmp(tag, "JOIN") == 0)
-		process_func = _net_handle_join;
-	else if (strcasecmp(tag, "LEAVE") == 0)
-		process_func = _net_handle_leave;
-	else if (strcasecmp(tag, "RESEAT") == 0)
-		process_func = _net_handle_reseat;
-	else if (strcasecmp(tag, "LAUNCH") == 0)
-		process_func = _net_handle_launch;
-	else if (strcasecmp(tag, "TABLE") == 0)
-		process_func = _net_handle_table;
-	else if (strcasecmp(tag, "SEAT") == 0)
-		process_func = _net_handle_seat;
-	else if (strcasecmp(tag, "DESC") == 0)
-		process_func = _net_handle_desc;
-	else if (strcasecmp(tag, "MOTD") == 0)
-		process_func = _net_handle_motd;
-	else if (strcasecmp(tag, "PONG") == 0)
-		process_func = _net_handle_pong;
-	else if (strcasecmp(tag, "PING") == 0)
-		process_func = _net_handle_ping;
-	else if (strcmp(tag, "TLS_START") == 0)
-		process_func = _net_handle_tls_start;
-	else
-		process_func = NULL;
+	for (i = 0; i < ARRAY_SIZE(tags); i++) {
+		if (strcasecmp(tags[i].tag, tag) == 0) {
+			process_func = tags[i].process_func;
+			break;
+		}
+	}
 
-	return ggz_xmlelement_new(tag, attrs, process_func, NULL);
+	return ggz_xmlelement_new(tag, attrs, (ggzxmlfunc)process_func, NULL);
 }
 
 
@@ -1548,6 +1631,87 @@ static void _net_handle_chat(GGZNetIO *net, GGZXMLElement *element)
 }
 
 
+/* Functions for <ADMIN> tag */
+static void _net_handle_admin(GGZNetIO *net, GGZXMLElement *element)
+{
+	const char *action_str, *player, *reason;
+	GGZAdminType type;
+
+	if (!element) return;
+	if (!check_playerconn(net, "admin")) return;
+
+	/* Grab admin data from tag */
+	action_str = ggz_xmlelement_get_attr(element, "ACTION");
+	player = ggz_xmlelement_get_attr(element, "PLAYER");
+
+	reason = ggz_xmlelement_get_data(element);
+
+	if (!action_str) {
+		return;
+	} else if (!player) {
+		return;
+	} else {
+		type = ggz_string_to_admintype(action_str);
+	}
+
+	player_room_admin(net->client->data, type, player, reason);
+
+	if (reason) {
+		ggz_free(reason);
+	}
+}
+
+
+/* Functions for <ADMINPERM> tag */
+static void _net_handle_adminperm(GGZNetIO *net, GGZXMLElement *element)
+{
+	const char *player, *permstr, *valuestr;
+	bool value;
+	GGZPerm perm;
+
+	if (!element) return;
+	if (!check_playerconn(net, "admin")) return;
+
+	player = ggz_xmlelement_get_attr(element, "PLAYER");
+	permstr = ggz_xmlelement_get_attr(element, "PERM");
+	valuestr = ggz_xmlelement_get_attr(element, "VALUE");
+
+	perm = ggz_string_to_perm(permstr);
+	value = str_to_bool(valuestr, false);
+
+	player_perms_admin(net->client->data, player, perm, value);
+}
+
+
+/* Functions for <REASON> tag */
+static void _net_handle_reason(GGZNetIO *net, GGZXMLElement *element)
+{
+	char *reason;
+	const char *parent_tag;
+	GGZXMLElement *parent;
+
+	if (!element) return;
+
+	/* Get parent off top of stack */
+	parent = ggz_stack_top(net->stack);
+	if (!parent) {
+		/* If there are no elements above us, it's a protocol error */
+		_net_send_result(net, "protocol", E_BAD_OPTIONS);
+		return;
+	}
+
+	parent_tag = ggz_xmlelement_get_tag(parent);
+	if (strcasecmp(parent_tag, "ADMIN") != 0) {
+		_net_send_result(net, "protocol", E_BAD_OPTIONS);
+		return;
+	}
+
+	reason = ggz_strdup(ggz_xmlelement_get_text(element));
+
+	ggz_xmlelement_set_data(parent, reason);
+}
+
+
 /* Functions for <INFO> tag */
 static void _net_handle_info(GGZNetIO *net, GGZXMLElement *element)
 {
@@ -1746,20 +1910,28 @@ static void _net_handle_table(GGZNetIO *net, GGZXMLElement *element)
 			break;
 		case GGZ_SEAT_RESERVED:
 			/* We verify that this is a real,
-			   registered player name. */
+			   registered player name.  Note seat->name may be NULL
+			   if the client doesn't send a name. */
 			/* FIXME: This should be done elsewhere. */
 			snprintf(player.handle, MAX_USER_NAME_LEN+1,
 				 "%s", seat->name ? seat->name : "");
+#if 0 /* We used to disallow nonregistered reservations, but this seems
+	 to be a desired feature. */
 			if (ggzdb_player_get(&player) != 0) {
 				/* This is some kind of error...but for now we
 				   just cover it up. */
 				dbg_msg(GGZ_DBG_TABLE,
 					"Invalid name '%s' sent for reserved "
 					"seat.  Changing it to an open seat.",
-					seat->name);
+					player.handle);
 				seat_type = GGZ_SEAT_OPEN;
+			} else {
+#endif
+				strcpy(table->seat_names[seat->index],
+				       player.handle);
+#if 0
 			}
-			strcpy(table->seat_names[seat->index], seat->name);
+#endif
 			break;
 		case GGZ_SEAT_PLAYER:
 			/* This is valid for an UPDATE but not a LAUNCH.
@@ -1966,10 +2138,11 @@ static int str_to_int(const char *str, int dflt)
 {
 	int val;
 
-	if (!str || sscanf(str, "%d", &val) < 1)
-		return dflt;
+	if (!str) return dflt;
+	if (sscanf(str, "0x%x", &val) == 1) return val;
+	if (sscanf(str, "%d", &val) == 1) return val;
 
-	return val;
+	return dflt;
 }
 
 
@@ -2031,6 +2204,7 @@ static GGZReturn _net_send_seat(GGZNetIO *net, GGZTableSeat *seat)
 {
 	const char *type_str = ggz_seattype_to_string(seat->type);
 	char *name = NULL;
+	GGZReturn ret;
 
 	switch (seat->type) {
 	case GGZ_SEAT_RESERVED:
@@ -2048,11 +2222,16 @@ static GGZReturn _net_send_seat(GGZNetIO *net, GGZTableSeat *seat)
 	if (seat->type == GGZ_SEAT_BOT && seat->name[0] != '\0')
 		name = seat->name;
 	
-	if (name)
-		return _net_send_line(net,
+	if (name) {
+		const char *name_unquoted = ggz_xml_escape(name);
+
+		ret = _net_send_line(net,
 				      "<SEAT NUM='%d' TYPE='%s'>%s</SEAT>",
-				      seat->index, type_str, name);
-	else
+				      seat->index, type_str, name_unquoted);
+
+		ggz_free(name_unquoted);
+		return ret;
+	} else
 		return _net_send_line(net, "<SEAT NUM='%d' TYPE='%s'/>", 
 				      seat->index, type_str);
 }
@@ -2142,6 +2321,8 @@ static GGZReturn _net_send_line(GGZNetIO *net, char *line, ...)
 	va_end(ap);
 	strcat(buf, "\n");
 
+	_net_dump_data(net, buf, strlen(buf), true);
+
 	if (ggz_tls_write(net->fd, buf, strlen(buf)) < 0)
 		return GGZ_ERROR;
 	else
