@@ -4,7 +4,7 @@
  * Project: GGZ Server
  * Date: 10/15/99
  * Desc: Parse command-line arguments and conf file
- * $Id: parse_opt.c 7861 2006-02-13 07:02:50Z josef $
+ * $Id: parse_opt.c 8746 2006-12-23 21:35:25Z jdorje $
  *
  * Copyright (C) 1999-2002 Brent Hendricks.
  *
@@ -52,7 +52,7 @@
 /* Private file parsing functions */
 static void get_config_options(int);
 static void parse_game(char *, char *);
-static void parse_room(char *, char *);
+static void parse_room(char *, char *, int);
 static int parse_gselect(const struct dirent *);
 static int parse_rselect(const struct dirent *);
 static unsigned parse_log_types(int, char **);
@@ -151,13 +151,43 @@ static char **_ggz_string_to_list(const char *s, const char *sep)
 #else
 #define SPEC_HOWL 0
 #endif
+#ifdef WITH_AVAHI
+#define SPEC_AVAHI 1
+#else
+#define SPEC_AVAHI 0
+#endif
+#ifdef HAVE_INOTIFY
+#define SPEC_INOTIFY 1
+#else
+#define SPEC_INOTIFY 0
+#endif
+#ifdef WITH_FAM
+#define SPEC_FAM 1
+#else
+#define SPEC_FAM 0
+#endif
 
 static void dump_specs(void)
 {
 	int tls = ggz_tls_support_query();
+	const char *tlsname = ggz_tls_support_name();
+	char tlsstring[32], dbstring[32];
 
 	parse_conf_file();
 
+	if(tls)
+	{
+		snprintf(tlsstring, sizeof(tlsstring), "yes (%s)", tlsname);
+	}
+	if(!strcmp(DATABASE_TYPE, "dbi"))
+	{
+		snprintf(dbstring, sizeof(dbstring), " [using dbd::%s]", opt.dbtype);
+	}
+	else
+	{
+		strcpy(dbstring, "");
+	}
+
 	printf("GGZ Gaming Zone server (ggzd) specifications\n");
 	printf("Version: %s\n", VERSION);
 	printf("Protocol version: %i\n", GGZ_CS_PROTO_VERSION);
@@ -165,13 +195,16 @@ static void dump_specs(void)
 		GGZDCONFDIR,
 		(opt.conf_valid ? "ok" : "not ok"));
 	printf("Debugging: %s\n", (SPEC_DEBUG ? "yes" : "no"));
-	printf("Database backend: %s\n", SPEC_DB);
+	printf("Database backend: %s%s\n", SPEC_DB, dbstring);
 	printf("Zeroconf support: %s [%s]\n",
-		(SPEC_HOWL ? "yes" : "no"),
+		(SPEC_HOWL ? "yes (howl)" : (SPEC_AVAHI ? "yes (avahi)" : "no")),
 		(opt.conf_valid ? (opt.announce_lan ? "used" : "not used") : "unknown"));
 	printf("TLS support: %s [%s]\n",
-		(tls ? "yes" : "no"),
+		(tls ? tlsstring : "no"),
 		(opt.conf_valid ? (opt.tls_use ? "used" : "not used") : "unknown"));
+	printf("Reconfiguration support: %s [%s]\n",
+		(SPEC_INOTIFY ? "yes (inotify)" : (SPEC_FAM ? "yes (fam)" : "no")),
+		(opt.conf_valid ? (opt.reconfigure_rooms ? "used" : "not used") : "unknown" ));
 }
 
 /* Parse command-line options */
@@ -194,7 +227,6 @@ void parse_args(int argc, char *argv[])
 				opt.local_conf = optarg;
 				break;
 			case 'l':
-				/*log_info.log_types = atoi(optarg);*/
 				t_list = _ggz_string_to_list(optarg, ",");
 				t_count = 0;
 				while((t_list) && (t_list[t_count]))
@@ -270,9 +302,9 @@ void parse_conf_file(void)
 
 	/* Add any defaults which were not config'ed */
 
-	/* If no game_dir, default it to GAMEDIR */
-	if(!opt.game_dir)
-		opt.game_dir = ggz_strdup(GAMEDIR);
+	/* If no game_exec_dir, default it to GGZDEXECMODDIR */
+	if(!opt.game_exec_dir)
+		opt.game_exec_dir = ggz_strdup(GGZDEXECMODDIR);
 
 	/* If no conf_dir, default it to GGZDCONFDIR */
 	if(!opt.conf_dir)
@@ -280,11 +312,7 @@ void parse_conf_file(void)
 
 	/* If no data_dir, default it to DATADIR */
 	if(!opt.data_dir)
-		opt.data_dir = ggz_strdup(DATADIR);
-
-	/* If no tmp_dir, default it to TMPDIR */
-	if(!opt.tmp_dir)
-		opt.tmp_dir = ggz_strdup(TMPDIR);
+		opt.data_dir = ggz_strdup(GGZDDATADIR);
 
 	/* If no main_port, default it to 5688 (or whatever) */
 	if(!opt.main_port)
@@ -299,6 +327,12 @@ void parse_conf_file(void)
 	/* Database defaults */
 	if(!opt.dbhashing)
 		opt.dbhashing = ggz_strdup("plain");
+
+	if(!opt.dbhashencoding)
+		opt.dbhashencoding = ggz_strdup("base64");
+
+	if(!opt.motd_file)
+		opt.motd_file = ggz_strdup("ggzd.motd");
 }
 
 
@@ -330,20 +364,27 @@ static void get_config_options(int ch)
 	opt.tls_key = ggz_conf_read_string(ch, "General", "EncryptionKey", NULL);
 
 	/* [Directories] */
-	opt.game_dir = ggz_conf_read_string(ch, "Directories", "GameDir", NULL);
+	opt.game_exec_dir = ggz_conf_read_string(ch, "Directories", "GameDir", NULL);
 	opt.conf_dir = ggz_conf_read_string(ch, "Directories", "ConfDir", NULL);
 	opt.data_dir = ggz_conf_read_string(ch, "Directories", "DataDir", NULL);
 
 	/* [Database] */
+	opt.dbtype = ggz_conf_read_string(ch, "General", "DatabaseType", NULL);
 	opt.dbhost = ggz_conf_read_string(ch, "General", "DatabaseHost", NULL);
 	opt.dbname = ggz_conf_read_string(ch, "General", "DatabaseName", NULL);
 	opt.dbusername = ggz_conf_read_string(ch, "General", "DatabaseUsername", NULL);
 	opt.dbpassword = ggz_conf_read_string(ch, "General", "DatabasePassword", NULL);
 	opt.dbhashing = ggz_conf_read_string(ch, "General", "DatabaseHashing", NULL);
+	opt.dbhashencoding = ggz_conf_read_string(ch, "General", "DatabaseHashEncoding", NULL);
 
 	/* Announcements in [General] */
 	opt.announce_lan = ggz_conf_read_int(ch, "General", "AnnounceLAN", 0);
-	opt.announce_metaserver = ggz_conf_read_int(ch, "General", "AnnounceMetaserver", 0);
+	opt.announce_metaserver = ggz_conf_read_string(ch, "General", "AnnounceMetaserver", NULL);
+	opt.metausername = ggz_conf_read_string(ch, "General", "AnnounceMetaserverUsername", NULL);
+	opt.metapassword = ggz_conf_read_string(ch, "General", "AnnounceMetaserverPassword", NULL);
+
+	/* Reconfiguration in [General] */
+	opt.reconfigure_rooms = ggz_conf_read_int(ch, "General", "ReconfigureRooms", 0);
 
 	/* [Games] */
 	ggz_conf_read_list(ch, "Games", "GameList", &g_count, &g_list);
@@ -468,6 +509,11 @@ void parse_game_files(void)
 			else
 				dbg_msg(GGZ_DBG_CONFIGURATION,
 					"Ignoring game %s", name);
+
+			free(namelist[i]);
+		}
+		if (num_games >= 0) {
+			free(namelist);
 		}
 	}
 
@@ -520,6 +566,8 @@ static void parse_game(char *name, char *dir)
 	memset(&game_info, 0, sizeof(GameInfo));
 	pthread_rwlock_init(&game_info.lock, NULL);	
 
+	game_info.game = ggz_strdup(name);
+
 	/* [GameInfo] */
 	/* FIXME: this data is never free'd.  This isn't really a problem,
 	   but... */
@@ -531,6 +579,7 @@ static void parse_game(char *name, char *dir)
 		return;
 	}
 	strcpy(game_info.name, tmp);
+	ggz_free(tmp);
 	game_info.version = ggz_conf_read_string(ch, "GameInfo",
 						 "Version", "");
 	game_info.desc = ggz_conf_read_string(ch, "GameInfo",
@@ -543,20 +592,21 @@ static void parse_game(char *name, char *dir)
 	/* [LaunchInfo] */
 	ggz_conf_read_list(ch, "LaunchInfo", "ExecutablePath",
 			   &num_args, &game_info.exec_args);
-	if (!game_info.exec_args[0]) {
+	if (!game_info.exec_args || !game_info.exec_args[0]) {
 		err_msg("Missing ExecutablePath for game %s.",
 			game_info.name);
 		/* This leaves some memory leak, but it's acceptable under
 		   the circumstances. */
 		return;
 	}
-	/* If there's no absolute path given, we prepend the game_dir. */
+	/* If there's no absolute path given, we prepend the game_exec_dir. */
 	if (game_info.exec_args[0][0] != '/') {
 		char *new_exec;
-		len = strlen(game_info.exec_args[0])+strlen(opt.game_dir)+2;
+		len = strlen(game_info.exec_args[0])
+		  + strlen(opt.game_exec_dir) + 2;
 		new_exec = ggz_malloc(len);
 		snprintf(new_exec, len, "%s/%s",
-			 opt.game_dir, game_info.exec_args[0]);
+			 opt.game_exec_dir, game_info.exec_args[0]);
 		ggz_free(game_info.exec_args[0]);
 		game_info.exec_args[0] = new_exec;
 	}
@@ -620,8 +670,8 @@ static void parse_game(char *name, char *dir)
 			snprintf(db_pe.handle, sizeof(db_pe.handle), "%s",
 				game_info.named_bots[i][0]);
 			db_status = ggzdb_player_get(&db_pe);
-			if ((db_pe.perms & PERMS_CHAT_BOT)
-			&& (db_status != GGZDB_ERR_NOTFOUND)) {
+			if ((db_pe.perms & (1 << GGZ_PERM_CHAT_BOT))
+			    && (db_status != GGZDB_ERR_NOTFOUND)) {
 				dbg_msg(GGZ_DBG_CONFIGURATION,
 					"named_bot <%s> registered already.",
 					game_info.named_bots[i][0]);
@@ -629,7 +679,9 @@ static void parse_game(char *name, char *dir)
 			}
 
 			db_pe.user_id = ggzdb_player_next_uid();
-			db_pe.perms = PERMS_DEFAULT_SETTINGS | PERMS_CHAT_BOT;
+			ggz_perms_init_from_list(&db_pe.perms,
+					     perms_default, num_perms_default);
+			db_pe.perms |= (1 << GGZ_PERM_CHAT_BOT);
 			db_pe.last_login = time(NULL);
 			snprintf(db_pe.handle, sizeof(db_pe.handle), "%s",
 				game_info.named_bots[i][0]);
@@ -672,7 +724,7 @@ void parse_room_files(void)
 	/* Setup our directory to "conf_dir/rooms/" */
 	snprintf(dir, sizeof(dir), "%s%s", opt.conf_dir, suffix);
 
-	parse_room("entry", dir);
+	parse_room("entry", dir, 0);
 
 	if(r_count > 0) {
 		/* Go through all rooms explicitly included in the add list */
@@ -680,7 +732,7 @@ void parse_room_files(void)
 		for(i=0; i<r_count; i++)
 			/* Add everything, but don't readd entry room */
 			if(strcmp(r_list[i], "entry"))
-				parse_room(r_list[i], dir);
+				parse_room(r_list[i], dir, 0);
 	} else {
 		/* Scan for all .room files in dir */
 		dbg_msg(GGZ_DBG_CONFIGURATION, "Adding all rooms in %s", dir);
@@ -692,8 +744,10 @@ void parse_room_files(void)
 			name[strlen(name)-5] = '\0';
 
 			/* Don't readd the entry room */
-			if(!strcmp(name, "entry"))
+			if(!strcmp(name, "entry")){
+				free(namelist[i]);
 				continue;
+			}
 			/* Check to see if this game is on the ignore list */
 			addit=1;
 			for(j=0; j>r_count; j--) {
@@ -704,10 +758,15 @@ void parse_room_files(void)
 
 			/* Add it if it's not on the ignore list */
 			if(addit)
-				parse_room(name, dir);
+				parse_room(name, dir, 0);
 			else
 				dbg_msg(GGZ_DBG_CONFIGURATION,
 					"Ignoring room %s", name);
+
+			free(namelist[i]);
+		}
+		if (num_rooms >= 0) {
+			free(namelist);
 		}
 	}
 
@@ -729,7 +788,7 @@ void parse_room_files(void)
 
 
 /* Parse a single room file, adding it's values to the room table */
-static void parse_room(char *name, char *dir)
+static void parse_room(char *name, char *dir, int announce)
 {
 	char fname[strlen(name) + strlen(dir) + 7];
 	char *strval;
@@ -753,6 +812,7 @@ static void parse_room(char *name, char *dir)
 	num = room_info.num_rooms - 1;
 	rooms[num].game_type = -2;
 	rooms[num].perms = 0;
+	rooms[num].room = ggz_strdup(name);
 
 	/* [RoomInfo] */
 	rooms[num].name = ggz_conf_read_string(ch, "RoomInfo", "Name", NULL);
@@ -780,9 +840,9 @@ static void parse_room(char *name, char *dir)
 				      "EntryRestriction", NULL);
 	if(strval) {
 		if(!strcasecmp(strval, "Admin"))
-			rooms[num].perms = PERMS_ROOMS_ADMIN;
+			rooms[num].perms = 1 << GGZ_PERM_ROOMS_ADMIN;
 		else if(!strcasecmp(strval, "Registered"))
-			rooms[num].perms = PERMS_ROOMS_LOGIN;
+			rooms[num].perms = 1 << GGZ_PERM_ROOMS_LOGIN;
 		else if(!strcasecmp(strval, "None"))
 			rooms[num].perms = 0;
 		else
@@ -813,6 +873,36 @@ static void parse_room(char *name, char *dir)
 		rooms[num].game_type = -1;
 	}
 
+	if (rooms[num].game_type != -1) {
+		char **exec_args;
+		int num_args, i, j;
+		GameInfo *gtype = &game_types[rooms[num].game_type];
+
+		ggz_conf_read_list(ch, "RoomInfo", "ExecutableArgs",
+				   &num_args, &exec_args);
+		for (i = 0; gtype->exec_args[i]; i++) {
+
+		}
+		rooms[num].exec_args
+		  = ggz_malloc((num_args + i + 1)
+			       * sizeof(*rooms[num].exec_args));
+		for (i = 0; gtype->exec_args[i]; i++) {
+			rooms[num].exec_args[i]
+			  = ggz_strdup(gtype->exec_args[i]);
+		}
+		for (j = 0; j < num_args; j++) {
+		  rooms[num].exec_args[i + j] = exec_args ? exec_args[j] : NULL;
+		}
+		rooms[num].exec_args[i + j] = NULL;
+#if 0
+		printf("%s : %d args\n", rooms[num].name, i + j);
+		for (i = 0; rooms[num].exec_args[i]; i++) {
+		  printf("  %s\n", rooms[num].exec_args[i]);
+		}
+#endif
+		if (exec_args) ggz_free(exec_args);
+	}
+
 	rooms[num].players = ggz_malloc(rooms[num].max_players
 					* sizeof(GGZPlayer*));
 	if (rooms[num].max_tables > 0)
@@ -820,6 +910,54 @@ static void parse_room(char *name, char *dir)
 					       * sizeof(GGZTable*));
 	else
 		rooms[num].tables = NULL;
+
+	if (announce) {
+		room_add(num);
+	}
+}
+
+
+void parse_room_change(const char *room)
+{
+	const char *suffix = "/rooms/";
+	char dir[strlen(opt.conf_dir) + strlen(suffix) + 1];
+	char roomname[strlen(room) + 1];
+	int i;
+
+	/* Setup our directory to "conf_dir/rooms/" */
+	snprintf(dir, sizeof(dir), "%s%s", opt.conf_dir, suffix);
+
+	snprintf(roomname, sizeof(roomname), "%s", room);
+	roomname[strlen(roomname) - 5] = '\0';
+
+	for(i = 1; i < room_info.num_rooms; i++)
+	{
+		if(!strcmp(rooms[i].room, roomname))
+		{
+			if(rooms[i].removal_done)
+			{
+				/* 'k, we can add it then? */
+				/* this code should never be reached, since removal_done rooms */
+				/* have lost their .room association */
+			}
+			else if(rooms[i].removal_pending)
+			{
+				/* FIXME: FAM does indeed send removal requests twice */
+				/* so we must discard additions during this time, too */
+
+				dbg_msg(GGZ_DBG_CONFIGURATION, "Room %s subject of removal", roomname);
+				return;
+			}
+			else
+			{
+				dbg_msg(GGZ_DBG_CONFIGURATION, "Removing room %s", roomname);
+				room_remove(i);
+				return;
+			}
+		}
+	}
+
+	parse_room(roomname, dir, 1);
 }
 
 
