@@ -4,7 +4,7 @@
  * Project: GGZ
  * Date: 10/18/99
  * Desc: Protocol enumerations, etc.
- * $Id: protocols.h 6145 2004-07-17 16:20:35Z josef $
+ * $Id: protocols.h 8763 2006-12-27 10:02:33Z jdorje $
  *
  * Copyright (C) 1999 Brent Hendricks.
  *
@@ -31,31 +31,38 @@
    itself (in ggzd and ggzcore). */
 #define GGZ_CS_PROTO_VERSION 10
 
+/* Some code is used by several of the following update types.
+   Therefore, they must not overlap. */
+
 /* Update opcodes */
 typedef enum {
-	GGZ_PLAYER_UPDATE_ADD, /* player enters room */
-	GGZ_PLAYER_UPDATE_DELETE, /* player leaves room */
-	GGZ_PLAYER_UPDATE_LAG, /* player's lag status changed */
-	GGZ_PLAYER_UPDATE_STATS /* player's stats changed */
+	GGZ_PLAYER_UPDATE_ADD = 1, /* player enters room */
+	GGZ_PLAYER_UPDATE_DELETE = 2, /* player leaves room */
+	GGZ_PLAYER_UPDATE_LAG = 3, /* player's lag status changed */
+	GGZ_PLAYER_UPDATE_STATS = 4, /* player's stats changed */
+	GGZ_PLAYER_UPDATE_PERMS = 5 /* player's permissions changed */
 } GGZPlayerUpdateType;
 
 /* Update opcodes */
 typedef enum {
-	GGZ_TABLE_UPDATE_ADD, /* table is added to room */
-	GGZ_TABLE_UPDATE_DELETE, /* table is deleted from room */
-	GGZ_TABLE_UPDATE_JOIN, /* player joins table */
-	GGZ_TABLE_UPDATE_LEAVE, /* player leaves table */
-	GGZ_TABLE_UPDATE_SEAT, /* misc. table seat change */
-	GGZ_TABLE_UPDATE_SPECTATOR_LEAVE, /* spectator leaves table */
-	GGZ_TABLE_UPDATE_SPECTATOR_JOIN, /* spectator joins table */
-	GGZ_TABLE_UPDATE_STATE, /* table state changes */
-	GGZ_TABLE_UPDATE_DESC, /* table desc changes */
-	GGZ_TABLE_UPDATE_RESIZE /* change size of table */
+	GGZ_TABLE_UPDATE_ADD = 10, /* table is added to room */
+	GGZ_TABLE_UPDATE_DELETE = 11, /* table is deleted from room */
+	GGZ_TABLE_UPDATE_JOIN = 12, /* player joins table */
+	GGZ_TABLE_UPDATE_LEAVE = 13, /* player leaves table */
+	GGZ_TABLE_UPDATE_SEAT = 14, /* misc. table seat change */
+	GGZ_TABLE_UPDATE_SPECTATOR_LEAVE = 15, /* spectator leaves table */
+	GGZ_TABLE_UPDATE_SPECTATOR_JOIN = 16, /* spectator joins table */
+	GGZ_TABLE_UPDATE_STATE = 17, /* table state changes */
+	GGZ_TABLE_UPDATE_DESC = 18, /* table desc changes */
+	GGZ_TABLE_UPDATE_RESIZE = 19 /* change size of table */
 } GGZTableUpdateType;
 
 /* Update opcodes. */
 typedef enum {
-	GGZ_ROOM_UPDATE_PLAYER_COUNT /* Number of players in room changes. */
+	GGZ_ROOM_UPDATE_ADD = 20, /* room is added to server */
+	GGZ_ROOM_UPDATE_DELETE = 21, /* room is deleted from server */
+	GGZ_ROOM_UPDATE_CLOSE = 22, /* room is deleted soon */
+	GGZ_ROOM_UPDATE_PLAYER_COUNT = 23 /* Number of players in room changes */
 } GGZRoomUpdateType;
 
 typedef enum {
