@@ -4,7 +4,7 @@
  * Project: GGZ Server
  * Date: 3/20/00
  * Desc: Functions for interfacing with room and chat facility
- * $Id: room.c 6729 2005-01-18 18:23:46Z jdorje $
+ * $Id: room.c 8763 2006-12-27 10:02:33Z jdorje $
  *
  * Copyright (C) 2000 Brent Hendricks.
  *
@@ -71,14 +71,17 @@ GGZPlayerHandlerStatus room_list_send(GGZPlayer* player, int req_game,
 
 	/* We don't need to lock anything because CURRENTLY the room count  */
 	/* and options can change ONLY before threads are in existence	    */
-		
+
+	/* Update: actual now that rooms can change, must be revisited */
+	/* In particular, which rooms should be sent, does order matter etc? */
+
 	/* Don't send list if they're not logged in */
 	if (player->login_status == GGZ_LOGIN_NONE) {
 		if (net_send_room_list_error(player->client->net, E_NOT_LOGGED_IN) < 0)
 			return GGZ_REQ_DISCONNECT;
 		return GGZ_REQ_FAIL;
 	}
-	
+
 	pthread_rwlock_rdlock(&state.lock);
 	max = state.types;
 	pthread_rwlock_unlock(&state.lock);	
@@ -93,21 +96,28 @@ GGZPlayerHandlerStatus room_list_send(GGZPlayer* player, int req_game,
 	/* First we have to figure out how many rooms to announce  */
 	/* This is easy if a req_game filter hasn't been specified */
 	if(req_game != -1) {
-		for(i=0; i<room_info.num_rooms; i++)
+		for(i = 0; i < room_get_num_rooms(); i++)
+			if(room_will_be_removed(i))
+				continue;
 			if(req_game == rooms[i].game_type)
 				count++;
 	} else
-		count = room_info.num_rooms;
+		count = room_get_count_rooms();
 
 	/* Do da opcode, and announce our count */
 	if (net_send_room_list_count(player->client->net, count) < 0)
 		return GGZ_REQ_DISCONNECT;
 
 	/* Send off all the room announcements */
-	for(i=0; i<room_info.num_rooms; i++)
+	for(i = 0; i < room_get_num_rooms(); i++)
 		if(req_game == -1 || req_game == rooms[i].game_type) {
 			GGZReturn result;
 
+			/* Skip removal-pending rooms (includes removal-done) */
+			if (room_will_be_removed(i)) {
+				continue;
+			}
+
 			/* We get a read-lock before sending because we
 			 * send the number of players which may change.  All
 			 * other data sent is unchanging and doesn't require
@@ -148,12 +158,14 @@ void room_initialize(void)
 {
 	dbg_msg(GGZ_DBG_ROOM, "Initializing room array");
 
-	room_info.num_rooms=1;
+	room_info.num_rooms = 1;
+	room_info.count_rooms = 1;
+	pthread_rwlock_init(&room_info.lock, NULL);
 
 	/* Allocate a big enough array to hold all our first room */
 	rooms = ggz_malloc(room_info.num_rooms * sizeof(RoomStruct));
 
-	/* Initialize the chat_tail and lock */
+	/* Initialize the room_tail and lock */
 	rooms[0].event_tail = NULL;
 	pthread_rwlock_init(&rooms[0].lock, NULL);
 #ifdef DEBUG
@@ -163,25 +175,124 @@ void room_initialize(void)
 
 
 /* Initialize an additional room */
+/* Called from control thread only */
 void room_create_additional(void)
 {
-	/* Right now this is only used at startup, so we don't lock anything */
+	int room_id;
+
 	dbg_msg(GGZ_DBG_ROOM, "Creating a new room");
 
-	room_info.num_rooms++;
+	room_id = room_info.num_rooms;
 
 	/* Reallocate the room's array */
-	rooms = ggz_realloc(rooms, room_info.num_rooms * sizeof(RoomStruct));
+	/* FIXME: reallocation might create wholly new array? */
+	rooms = ggz_realloc(rooms, (room_id + 1) * sizeof(RoomStruct));
 
-	/* Initialize the chat_tail and lock on the new one */
-	rooms[room_info.num_rooms-1].player_count = 0;
-	rooms[room_info.num_rooms-1].last_player_change = 0;
-	rooms[room_info.num_rooms-1].table_count = 0;
-	rooms[room_info.num_rooms-1].event_tail = NULL;
-	pthread_rwlock_init(&rooms[room_info.num_rooms-1].lock, NULL);
+	/* Initialize the room_tail and lock on the new one */
+	rooms[room_id].player_count = 0;
+	rooms[room_id].last_player_change = 0;
+	rooms[room_id].table_count = 0;
+	rooms[room_id].event_tail = NULL;
+	pthread_rwlock_init(&rooms[room_id].lock, NULL);
 #ifdef DEBUG
-	rooms[room_info.num_rooms-1].event_head = NULL;
+	rooms[room_id].event_head = NULL;
 #endif
+	rooms[room_id].removal_pending = 0;
+	rooms[room_id].removal_done = 0;
+
+	/* Finally, make it available to iterations over all rooms */
+	pthread_rwlock_wrlock(&room_info.lock);
+	room_info.num_rooms++;
+	room_info.count_rooms++;
+	pthread_rwlock_unlock(&room_info.lock);
+}
+
+/* Announce a room which was added */
+/* Only called from control thread */
+void room_add(int room)
+{
+	int roomnum = room_info.num_rooms, i;
+
+	for (i = 0; i < roomnum; i++) {
+		if (rooms[i].removal_done)
+			continue;
+
+		room_update_event("", GGZ_ROOM_UPDATE_ADD, i, room);
+	}
+
+#if 0
+	/* FIXME: This needs to go into the clients! */
+	chat_server_2_player(NULL, "Reconfiguration: A new room has been added!");
+#endif
+}
+
+
+/* Mark a room to be subject of removal */
+/* FIXME: together with room_create_additional, should return error status? */
+/* Only called from control thread (but we lock for writing) */
+void room_remove(int room)
+{
+	int i;
+
+	/* actual rooms really start with index 1 (= entry room) */
+	if (room <= 0 || room > room_info.num_rooms - 1)
+		return;
+	if (rooms[room].removal_pending)
+		return;
+
+	pthread_rwlock_wrlock(&rooms[room].lock);
+	rooms[room].removal_pending = 1;
+	pthread_rwlock_unlock(&rooms[room].lock);
+	pthread_rwlock_wrlock(&room_info.lock);
+	room_info.count_rooms--;
+	pthread_rwlock_unlock(&room_info.lock);
+
+	/* We can kill it off right away if nobody's in there */
+	if (rooms[room].player_count == 0) {
+		room_remove_really(room);
+		return;
+	}
+
+	dbg_msg(GGZ_DBG_ROOM, "Marking room %i for removal", room);
+
+	/* TODO: need global event enqueue function (for server-wide events) */
+
+	for (i = 0; i < room_info.num_rooms; i++) {
+		if (rooms[i].removal_done)
+			continue;
+
+		room_update_event("", GGZ_ROOM_UPDATE_CLOSE, i, room);
+	}
+
+#if 0
+	/* FIXME: really send this? */
+	chat_server_2_player(NULL, "A room is about to close!");
+#endif
+}
+
+
+/* Called from control thread only */
+void room_remove_really(int old_room)
+{
+	int i;
+
+	if(rooms[old_room].player_count == 0
+	&& rooms[old_room].removal_pending)
+	{
+		pthread_rwlock_wrlock(&rooms[old_room].lock);
+		rooms[old_room].removal_done = 1;
+		pthread_rwlock_unlock(&rooms[old_room].lock);
+
+		dbg_msg(GGZ_DBG_ROOM, "Delete room %i entirely", old_room);
+
+		/* For notification, see ROOM_UPDATE_CLOSE */
+		for (i = 0; i < room_info.num_rooms; i++) {
+			room_update_event("", GGZ_ROOM_UPDATE_DELETE, i, old_room);
+		}
+#if 0 /* This spam message is not useful. */
+		chat_server_2_player(NULL, "A room was removed!");
+#endif
+	}
 }
 
 
@@ -190,7 +301,7 @@ GGZPlayerHandlerStatus room_handle_join(GGZPlayer* player, int room)
 {
 	int result;
 	char *rname;
-	
+
 	/* Check for silliness from the user */
 	if (player->table != -1 || player->launching) {
 		if (net_send_room_join(player->client->net, E_AT_TABLE) < 0)
@@ -203,8 +314,14 @@ GGZPlayerHandlerStatus room_handle_join(GGZPlayer* player, int room)
 			return GGZ_REQ_DISCONNECT;
 		return GGZ_REQ_FAIL;
 	}
-	
-	if(room > room_info.num_rooms || room < 0) {
+
+	if(room > room_get_num_rooms() || room < 0) {
+		if (net_send_room_join(player->client->net, E_BAD_OPTIONS) < 0)
+			return GGZ_REQ_DISCONNECT;
+		return GGZ_REQ_FAIL;
+	}
+
+	if(room_will_be_removed(room)) {
 		if (net_send_room_join(player->client->net, E_BAD_OPTIONS) < 0)
 			return GGZ_REQ_DISCONNECT;
 		return GGZ_REQ_FAIL;
@@ -239,7 +356,7 @@ static void update_room_stats(GGZPlayer *player, int game_type)
 
 	/* Don't show stats for guest players at all.  Later it may be
 	   that some games will want to let guest players have stats. */
-	if (!perms_check(player, PERMS_NO_STATS)
+	if (!perms_check(player, GGZ_PERM_NO_STATS)
 	    && game_type >= 0) {
 		pthread_rwlock_rdlock(&game_types[game_type].lock);
 		strcpy(stats.game, game_types[game_type].name);
@@ -250,7 +367,7 @@ static void update_room_stats(GGZPlayer *player, int game_type)
 	} else {
 		records = ratings = highscores = 0;
 	}
-	
+
 	if (records || ratings || highscores) {
 		strcpy(stats.player, player->name);
 		/* FIXME: doing a DB lookup while we have a write-lock
@@ -293,7 +410,7 @@ GGZClientReqError room_join(GGZPlayer* player, const int room)
 	/* Check for valid inputs */
 	if(old_room == room)
 		return E_OK;
-	if(room > room_info.num_rooms || room < -2)
+	if(room > room_get_num_rooms() || room < -2)
 		return E_BAD_OPTIONS;
 
 	/* Process queued messages unless they are connecting/disconnecting */
@@ -313,8 +430,8 @@ GGZClientReqError room_join(GGZPlayer* player, const int room)
 	}
 
 	/* Check permissions to enter new room */
-	if(room != -1 && rooms[room].perms != 0 &&
-	   perms_check(player, rooms[room].perms) == PERMS_DENY) {
+	if (room != -1 && rooms[room].perms != 0
+	    && !perms_check(player, rooms[room].perms)) {
 		pthread_rwlock_unlock(&rooms[room].lock);
 		if(old_room != -1)
 			pthread_rwlock_unlock(&rooms[old_room].lock);
@@ -375,6 +492,10 @@ GGZClientReqError room_join(GGZPlayer* player, const int room)
 
 	room_notify_change(name, old_room, room);
 
+	/* Reconfiguration: last man leaving might kill room */
+	if(old_room != -1)
+		room_remove_really(old_room);
+
 	return E_OK;
 }
 
@@ -383,7 +504,7 @@ GGZClientReqError room_join(GGZPlayer* player, const int room)
 static void room_notify_change(char* name, const int old, const int new)
 {
 	dbg_msg(GGZ_DBG_ROOM, "%s moved from room %d to %d", name, old, new);
-		
+
 	if (old != -1) {
 		/* Send DELETE update to old room */
 		room_update_event(name, GGZ_PLAYER_UPDATE_DELETE, old, new);
@@ -416,10 +537,24 @@ static GGZEventFuncReturn room_event_callback(void* target_player,
 {
 	GGZPlayer *player = target_player;
 	GGZRoomEventData *event = data;
+	RoomStruct room;
+
+	if (event->opcode == GGZ_ROOM_UPDATE_CLOSE
+	|| event->opcode == GGZ_ROOM_UPDATE_DELETE) {
+		net_send_room_update(player->client->net, event->opcode, event->other_room, 0, NULL);
+		return GGZ_EVENT_OK;
+	}
+
+	if (event->opcode == GGZ_ROOM_UPDATE_ADD) {
+		room = rooms[event->other_room];
+		net_send_room_update(player->client->net, event->opcode, event->other_room, 0, &room);
+		return GGZ_EVENT_OK;
+	}
 
 	/* Don't deliver updates about ourself (except lag/stats) */
 	if (event->opcode != GGZ_PLAYER_UPDATE_LAG
 	    && event->opcode != GGZ_PLAYER_UPDATE_STATS
+	    && event->opcode != GGZ_PLAYER_UPDATE_PERMS
 	    && strcasecmp(event->player, player->name) == 0)
 		return GGZ_EVENT_OK;
 
@@ -427,13 +562,62 @@ static GGZEventFuncReturn room_event_callback(void* target_player,
 				   event->player, player_get_room(player),
 				   event->other_room) < 0)
 		return GGZ_EVENT_ERROR;
-	
+
 	return GGZ_EVENT_OK;
 }
 
 
+/* Only use in non-control thread since locking is expensive */
 int room_get_num_rooms(void)
 {
-	/* FIXME: Lock this one day when we have dynamic rooms */
-	return room_info.num_rooms;
+	int number;
+
+	pthread_rwlock_rdlock(&room_info.lock);
+	number = room_info.num_rooms;
+	pthread_rwlock_unlock(&room_info.lock);
+
+	return number;
 }
+
+/* Only use in non-control thread since locking is expensive */
+int room_get_count_rooms(void)
+{
+	int number;
+
+	pthread_rwlock_rdlock(&room_info.lock);
+	number = room_info.count_rooms;
+	pthread_rwlock_unlock(&room_info.lock);
+
+	return number;
+}
+
+/* This is always called from within loops over all rooms */
+/* Only use in non-control thread since locking is expensive */
+int room_is_removed(int room)
+{
+	int removed = 0;
+
+	pthread_rwlock_rdlock(&rooms[room].lock);
+	if (rooms[room].removal_done) {
+		removed = 1;
+	}
+	pthread_rwlock_unlock(&rooms[room].lock);
+
+	return removed;
+}
+
+/* This is always called from within loops over all rooms */
+/* Only use in non-control thread since locking is expensive */
+int room_will_be_removed(int room)
+{
+	int removed = 0;
+
+	pthread_rwlock_rdlock(&rooms[room].lock);
+	if (rooms[room].removal_pending) {
+		removed = 1;
+	}
+	pthread_rwlock_unlock(&rooms[room].lock);
+
+	return removed;
+}
+
