@@ -4,7 +4,7 @@
  * Project: GGZ Server
  * Date: 3/20/00
  * Desc: Functions for interfacing with the room and chat facility
- * $Id: room.h 5903 2004-02-11 03:27:12Z jdorje $
+ * $Id: room.h 8279 2006-06-27 07:29:39Z josef $
  *
  * Copyright (C) 2000 Brent Hendricks.
  *
@@ -44,6 +44,9 @@ typedef struct {
 	/* Room name */
 	char *name;			/* cleanup() */
 
+	/* Room file name (internal) */
+	char *room;			/* cleanup() */
+
 	/* Room description */
 	char *description;		/* cleanup() */
 
@@ -62,6 +65,11 @@ typedef struct {
 	/* Type of game played in this room */
 	int game_type;
 
+	/* Executable information: a NULL-terminated list of
+	   command-line arguments for the program. These are appended to
+	   the game type's arguments at execution time. */
+	char **exec_args; /* cleanup() */
+
 	/* Room entry restrictions */
 	unsigned int perms;	/* Set bits to equal perms in perms.h */
 
@@ -79,15 +87,23 @@ typedef struct {
 #ifdef DEBUG
 	GGZEvent *event_head;
 #endif
+
+	/* If set, room will be nuked when all players left (and then remains set!) */
+	int removal_pending;
+
+	/* If set, room is nuked (since we cannot really modify the rooms array) */
+	int removal_done;
 } RoomStruct;
 
 
 /* Information about all rooms */
 typedef struct {
-#if 0
-	pthread_rwlock_t lock;	/* Not inititalized or used yet */
-#endif
+	/* Used to protect against room additions (not removals!) */
+	pthread_rwlock_t lock;
+	/* All rooms including removal-pending and removal-done */
 	int num_rooms;
+	/* Number of active rooms */
+	int count_rooms;
 } RoomInfo;
 
 
@@ -96,11 +112,17 @@ extern RoomInfo room_info;
 
 void room_initialize(void);
 void room_create_additional(void);
+void room_add(int room);
+void room_remove(int room);
+void room_remove_really(int room);
 GGZClientReqError room_join(GGZPlayer* player, const int);
 GGZPlayerHandlerStatus room_list_send(GGZPlayer* player, int game,
                                              char verbose);
 GGZPlayerHandlerStatus room_handle_join(GGZPlayer* player, int room);
 int room_get_num_rooms(void);
+int room_get_count_rooms(void);
+int room_is_removed(int room);
+int room_will_be_removed(int room);
 
 GGZReturn room_update_event(const char *player,
 			    GGZPlayerUpdateType update,
