@@ -4,7 +4,7 @@
  * Project: GGZ Server
  * Date: 1/9/00
  * Desc: Functions for handling tables
- * $Id: table.c 7612 2005-11-07 10:39:49Z josef $
+ * $Id: table.c 8776 2006-12-30 10:14:45Z oojah $
  *
  * Copyright (C) 1999-2002 Brent Hendricks.
  *
@@ -301,6 +301,8 @@ static GGZReturn table_handler_launch(GGZTable* table)
 		return GGZ_ERROR;
 	}
 
+	log_create_table();
+
 	return GGZ_OK;
 }
 
@@ -360,7 +362,11 @@ static void* table_new_thread(void *index_ptr)
 	}
 	count = ++rooms[table->room].table_count;
 	/* Find empty spot in room */ 
-	for (i = 0; i < rooms[table->room].max_tables; i++)
+	/* FIXME - there are problems with table 0 locking up, probably
+	 * due to something being incorrectly initialised somewhere.
+	 * A 1 based index is better anyway.and will hopefully help us
+	 * find the problem more easily. Roger 20061230 */
+	for (i = 1; i < rooms[table->room].max_tables; i++)
 		if (rooms[table->room].tables[i] == NULL)
 			break;
 	rooms[table->room].tables[i] = table;
@@ -414,6 +420,7 @@ static GGZReturn table_start_game(GGZTable *table)
 #endif
 	char **args;
 	char *pwd;
+	char *game;
 	int type, i, num_seats;
 	GGZReturn status = GGZ_OK;
 	GGZSeat seat;
@@ -430,7 +437,8 @@ static GGZReturn table_start_game(GGZTable *table)
 	type = table->type;
 	pthread_rwlock_rdlock(&game_types[type].lock);
 	pwd = game_types[type].data_dir;
-	args = game_types[type].exec_args;
+	args = rooms[table->room].exec_args;
+	game = game_types[type].game;
 	pthread_rwlock_unlock(&game_types[type].lock);
 
 	/* Set a pointer to the table so we can get it back in the
@@ -438,8 +446,8 @@ static GGZReturn table_start_game(GGZTable *table)
 	ggzdmod_set_gamedata(table->ggzdmod, table);
 
 	/* Setup handlers for game module events */
-        ggzdmod_set_handler(table->ggzdmod, GGZDMOD_EVENT_STATE, &table_handle_state);
-        ggzdmod_set_handler(table->ggzdmod, GGZDMOD_EVENT_LOG, &table_log);
+	ggzdmod_set_handler(table->ggzdmod, GGZDMOD_EVENT_STATE, &table_handle_state);
+	ggzdmod_set_handler(table->ggzdmod, GGZDMOD_EVENT_LOG, &table_log);
 	ggzdmod_set_handler(table->ggzdmod, GGZDMOD_EVENT_GAMEREPORT,
 			    &table_game_report);
 	ggzdmod_set_handler(table->ggzdmod, GGZDMOD_EVENT_SAVEGAMEREPORT,
@@ -454,23 +462,24 @@ static GGZReturn table_start_game(GGZTable *table)
 			    &table_game_req_open);
 	ggzdmod_set_handler(table->ggzdmod, GGZDMOD_EVENT_ERROR, &table_error);
 	
-        /* Setup seats for game table */
+	/* Setup seats for game table */
 	num_seats = seats_num(table);
 	ggzdmod_set_num_seats(table->ggzdmod, num_seats);
-        for (i = 0; i < num_seats; i++) {
+	for (i = 0; i < num_seats; i++) {
 		seat.num = i;
 		seat.type = seats_type(table, i);
 		if (seat.type == GGZ_SEAT_RESERVED || seat.type == GGZ_SEAT_BOT)
 			seat.name = table->seat_names[i];
 		else
 			seat.name = NULL;
-		seat.fd = -1;
+			seat.fd = -1;
 		if (ggzdmod_set_seat(table->ggzdmod, &seat) < 0)
 			status = GGZ_ERROR;
-        }
+	}
 
 	/* And start the game */
-	ggzdmod_set_module(table->ggzdmod, pwd, args);
+	log_msg(GGZ_LOG_TABLES, "Launching table: %s", args[0]);
+	ggzdmod_set_module(table->ggzdmod, game, pwd, args);
 	if (ggzdmod_connect(table->ggzdmod) < 0)
 		status = GGZ_ERROR;
 
@@ -573,6 +582,7 @@ void table_game_join(GGZTable *table, const char *name,
 		    && !strcasecmp(table->seat_names[i], name)) {
 			pthread_rwlock_wrlock(&table->lock);
 			table->seat_types[i] = GGZ_SEAT_OPEN;
+			table->seat_names[i][0] = '\0';
 			pthread_rwlock_unlock(&table->lock);
 			table_update_event_enqueue(table,
 						   GGZ_TABLE_UPDATE_SEAT,
@@ -608,6 +618,7 @@ void table_game_leave(GGZTable *table, const char *caller,
 	
 	pthread_rwlock_wrlock(&table->lock);
 	table->seat_types[num] = GGZ_SEAT_OPEN;
+	table->seat_names[num][0] = '\0';
 	pthread_rwlock_unlock(&table->lock);
 
 	empty = (seats_count(table, GGZ_SEAT_PLAYER) == 0);
@@ -655,6 +666,7 @@ void table_game_reseat(GGZTable *table,
 	case GGZ_RESEAT_STAND:
 		pthread_rwlock_wrlock(&table->lock);
 		table->seat_types[old_seat] = GGZ_SEAT_OPEN;
+		table->seat_names[old_seat][0] = '\0';
 		strcpy(table->spectators[new_seat], name);
 		pthread_rwlock_unlock(&table->lock);	
 		empty = (seats_count(table, GGZ_SEAT_PLAYER) == 0);
@@ -663,6 +675,7 @@ void table_game_reseat(GGZTable *table,
 	case GGZ_RESEAT_MOVE:
 		pthread_rwlock_wrlock(&table->lock);
 		table->seat_types[old_seat] = GGZ_SEAT_OPEN;
+		table->seat_names[old_seat][0] = '\0';
 		table->seat_types[new_seat] = GGZ_SEAT_PLAYER;
 		strcpy(table->seat_names[new_seat], name);
 		pthread_rwlock_unlock(&table->lock);
@@ -821,6 +834,7 @@ static void table_handle_state(GGZdMod *mod, GGZdModEvent event,
 #endif
 	GGZdModState cur = ggzdmod_get_state(mod);
 	GGZTableState origstate;
+	GGZTableState ggz_cur = table->state;
 
 	switch (cur) {
 	case GGZDMOD_STATE_WAITING:
@@ -829,17 +843,17 @@ static void table_handle_state(GGZdMod *mod, GGZdModEvent event,
 
 		/* FIXME: maybe we shouldn't keep track of table state
                    here any more? */
-		if (table->state != GGZ_TABLE_CREATED)
-			return;	
 		pthread_rwlock_wrlock(&table->lock);
 		table->state = GGZ_TABLE_WAITING;
 		pthread_rwlock_unlock(&table->lock);
-		
-		/* Signal owner that all is good */
-		table_launch_event(table->owner, E_OK, table->index);
+
+		if (ggz_cur == GGZ_TABLE_CREATED) {
+			/* Signal owner that all is good */
+			table_launch_event(table->owner, E_OK, table->index);
 	
-		/* Trigger a table update in this room */
-		table_event_enqueue(table, GGZ_TABLE_UPDATE_ADD);
+			/* Trigger a table update in this room */
+			table_event_enqueue(table, GGZ_TABLE_UPDATE_ADD);
+		}
 		return;
 		
 	case GGZDMOD_STATE_PLAYING:
@@ -961,7 +975,7 @@ static void table_game_req_num_seats(GGZdMod *ggzdmod, GGZdModEvent event,
 		table_change_num_seats(table, num_seats);
 		for (seat = old_seats; seat < num_seats; seat++) {
 			table->seat_types[seat] = GGZ_SEAT_OPEN;
-			/* Should we initialize the name? */
+			table->seat_names[seat][0] = '\0';
 		}
 		pthread_rwlock_unlock(&table->lock);
 
@@ -1066,6 +1080,7 @@ static void table_game_req_boot(GGZdMod *ggzdmod,
 		table->spectators[seat_num][0] = '\0';
 	} else {
 		table->seat_types[seat_num] = GGZ_SEAT_OPEN;
+		table->seat_names[seat_num][0] = '\0';
 	}
 	pthread_rwlock_unlock(&table->lock);
 
@@ -1149,6 +1164,7 @@ static void table_game_req_open(GGZdMod *ggzdmod,
 
 	pthread_rwlock_wrlock(&table->lock);
 	table->seat_types[seat_num] = GGZ_SEAT_OPEN;
+	table->seat_names[seat_num][0] = '\0';
 	pthread_rwlock_unlock(&table->lock);
 
 	/* FIXME: there's a bug whereby if a table requests an opening
@@ -1249,6 +1265,8 @@ static void table_remove(GGZTable* table)
 
 	/* Process any remaining events */
 	event_table_handle(table);
+
+	log_close_table();
 }
 
 
