@@ -4,7 +4,7 @@
  * Project: ggzdmod
  * Date: 10/14/01
  * Desc: GGZ game module functions
- * $Id: ggzdmod.c 7887 2006-03-07 09:56:51Z josef $
+ * $Id: ggzdmod.c 8567 2006-09-03 05:35:12Z jdorje $
  *
  * This file contains the backend for the ggzdmod library.  This
  * library facilitates the communication between the GGZ server (ggzd)
@@ -43,6 +43,7 @@
 #include <sys/socket.h>
 #include <sys/time.h>
 #include <sys/types.h>
+#include <sys/wait.h>
 #include <unistd.h>
 
 #include <ggz.h>
@@ -66,6 +67,7 @@
 static void call_handler(GGZdMod *ggzdmod, GGZdModEvent event, void *data);
 static int get_fd_max(GGZdMod * ggzdmod);
 static fd_set get_active_fd_set(GGZdMod * ggzdmod);
+static void _ggzdmod_set_game(GGZdMod * ggzdmod, const char *game);
 static void _ggzdmod_set_num_seats(GGZdMod * ggzdmod, int num_seats);
 static void _ggzdmod_set_max_num_spectators(GGZdMod * ggzdmod,
 					    int num_spectators);
@@ -90,6 +92,16 @@ static void spectator_free(GGZSpectator *spectator);
 /* Debugging function (see also ggzdmod_check) */
 static void seat_print(GGZdMod * ggzdmod, GGZSeat *seat);
 
+
+int ggzdmod_is_ggz_mode(void)
+{
+	char *ggzmode;
+
+	/* ggzdmod won't run in windows (windows doesn't have getenv()) */
+	ggzmode = getenv("GGZMODE");
+	return (ggzmode && strcmp(ggzmode, "true") == 0);
+}
+
 /* Invokes handlers for the specefied event */
 static void call_handler(GGZdMod * ggzdmod, GGZdModEvent event, void *data)
 {
@@ -238,6 +250,7 @@ GGZdMod *ggzdmod_new(GGZdModType type)
 
 	ggzdmod->pid = -1;
 	ggzdmod->argv = NULL;
+	ggzdmod->game = NULL;
 	/* Put any other necessary initialization here.  All fields
 	   should be initialized.  Note NULL may not necessarily be 0
 	   on all platforms. */
@@ -275,13 +288,16 @@ void ggzdmod_free(GGZdMod * ggzdmod)
 		ggz_free(ggzdmod->argv);
 	}
 
+	if(ggzdmod->game)
+		ggz_free(ggzdmod->game);
+
 	/* Free the object */
 	ggz_free(ggzdmod);
 }
 
 
 /* 
- * Accesor functions for GGZdMod
+ * Accessor functions for GGZdMod
  */
 
 /* The ggzdmod FD is the main ggzd<->game server communications socket. */
@@ -373,6 +389,11 @@ void* ggzdmod_get_gamedata(GGZdMod * ggzdmod)
 	return ggzdmod->gamedata;
 }
 
+static void _ggzdmod_set_game(GGZdMod *ggzdmod, const char *game)
+{
+	ggzdmod->game = ggz_strdup(game);
+}
+
 static void _ggzdmod_set_num_seats(GGZdMod *ggzdmod, int num_seats)
 {
 	int i, old_num;
@@ -405,17 +426,24 @@ static void _ggzdmod_set_num_seats(GGZdMod *ggzdmod, int num_seats)
 	ggzdmod->num_seats = num_seats;
 }
 
-
 char* ggzdmod_get_bot_class(GGZdMod *ggzdmod, const char *name)
 {
+#if 0 /* This doesn't belong in ggzdmod. */
 	int ch;
 	char *conffile;
 	const char *game;
 	char *botclass;
+	int len;
 
-	game = "tictactoe";
-	conffile = (char*)ggz_malloc(strlen(GGZDCONFDIR) + strlen("games") + strlen(game) + 7);
-	sprintf(conffile, "%s/games/%s.dsc", GGZDCONFDIR, game);
+	/* FIXME: find out which game we play here */
+	/* FIXME: Named bots mapping should probably be preloaded before */
+	/* FIXME: launch, e.g. in set_module() */
+	game = ggzdmod->game;
+	if(!game) return NULL;
+
+	len = strlen(GGZDCONFDIR) + strlen("games") + strlen(game) + 7;
+	conffile = ggz_malloc(len);
+	snprintf(conffile, len, "%s/games/%s.dsc", GGZDCONFDIR, game);
 	ch = ggz_conf_parse(conffile, GGZ_CONF_RDONLY);
 	if(ch < 0) return NULL;
 	ggz_free(conffile);
@@ -424,6 +452,8 @@ char* ggzdmod_get_bot_class(GGZdMod *ggzdmod, const char *name)
 	ggz_conf_close(ch);
 
 	return botclass;
+#endif
+	return NULL;
 }
 
 static void _ggzdmod_set_max_num_spectators(GGZdMod *ggzdmod,
@@ -465,7 +495,7 @@ int ggzdmod_set_num_seats(GGZdMod * ggzdmod, int num_seats)
 }
 
 void ggzdmod_set_module(GGZdMod * ggzdmod,
-                        const char *pwd, char **argv)
+                        const char *game, const char *pwd, char **argv)
 {
 	int i;
 
@@ -490,8 +520,9 @@ void ggzdmod_set_module(GGZdMod * ggzdmod,
 
 	ggz_debug("GGZDMOD", "Set %d arguments", i);
 	
-	ggzdmod->argv = ggz_malloc(sizeof(char*)*(i+1));
+	ggzdmod->argv = ggz_malloc(sizeof(*ggzdmod->argv) * (i + 1));
 	ggzdmod->pwd = ggz_strdup(pwd);
+	ggzdmod->game = ggz_strdup(game);
 	
 	for (i = 0; argv[i]; i++) 
 		ggzdmod->argv[i] = ggz_strdup(argv[i]);
@@ -516,6 +547,11 @@ int ggzdmod_set_handler(GGZdMod * ggzdmod, GGZdModEvent e,
 	if (e < 0) return -2;
 	if (e >= GGZDMOD_NUM_EVENTS) return -3;
 
+	/* Extra checks */
+	if (ggzdmod->type == GGZDMOD_GAME) {
+		if (e > GGZDMOD_EVENT_ERROR) return -4;
+	}
+
 	ggzdmod->handlers[e] = func;
 	return 0;
 }
@@ -581,10 +617,11 @@ int ggzdmod_set_seat(GGZdMod * ggzdmod, GGZSeat *seat)
 			   ggzdmod_disconnect - but this doesn't free the
 			   seat data.  Be careful with this! */
 			_ggzdmod_error(ggzdmod,
-				       "Error writing to game");
+				       "Error writing seat change to game");
 		}
 
-		/* We (GGZ) don't need the fd now */
+		/* We (GGZ) don't need the fd now (the exact same thing is
+		   done for spectators too, see ggzdmod_set_spectator). */
 		if (seat->type == GGZ_SEAT_PLAYER) {
 			close(seat->fd);
 			seat->fd = -1;
@@ -646,12 +683,17 @@ int ggzdmod_set_spectator(GGZdMod * ggzdmod, GGZSpectator *spectator)
 	    && ggzdmod->state != GGZDMOD_STATE_CREATED) {
 		if (_io_send_spectator_change(ggzdmod->fd, spectator) < 0) {
 			_ggzdmod_error(ggzdmod,
-				       "Error writing to game");
+				       "Error writing spectator "
+				       "change to game");
 			return -1;
 		}
 
-		/* We (GGZ) don't need the fd now */
-		spectator->fd = -1;
+		/* We (GGZ) don't need the fd now (the exact same thing is
+		   done for regular players too, see ggzdmod_set_seat). */
+		if (spectator->fd >= 0) {
+			close(spectator->fd);
+			spectator->fd = -1;
+		}
 	}
 
 	return _ggzdmod_set_spectator(ggzdmod, spectator);
@@ -691,9 +733,13 @@ int ggzdmod_reseat(GGZdMod * ggzdmod,
 	} else {
 		GGZSeatType new_type = ggzdmod_get_seat(ggzdmod,
 							new_seat).type;
+
+		/* I'm not sure if this is correct.  Does this allow seat
+		 * reservations to be stolen?  That would be bad. */
 		if (new_seat >= ggzdmod->num_seats
 		    || !(new_type == GGZ_SEAT_OPEN
-			 || new_type == GGZ_SEAT_RESERVED))
+			 || new_type == GGZ_SEAT_RESERVED
+			 || new_type == GGZ_SEAT_ABANDONED))
 			return -6;
 	}
 
@@ -1069,9 +1115,9 @@ static int send_game_launch(GGZdMod * ggzdmod)
 	GGZSeat *seat;
 		
 
-	if (_io_send_launch(ggzdmod->fd, ggzdmod->num_seats,
+	if (_io_send_launch(ggzdmod->fd, ggzdmod->game, ggzdmod->num_seats,
 			    ggzdmod->max_num_spectators) < 0) {
-		_ggzdmod_error(ggzdmod, "Error writing to game");
+		_ggzdmod_error(ggzdmod, "Error writing launch to game");
 		return -1;
 	}
 
@@ -1081,7 +1127,8 @@ static int send_game_launch(GGZdMod * ggzdmod)
 		
 		seat = ggz_list_get_data(entry);
 		if (_io_send_seat(ggzdmod->fd, seat) < 0) {
-			_ggzdmod_error(ggzdmod, "Error writing to game");
+			_ggzdmod_error(ggzdmod,
+				       "Error writing launch seats to game");
 			return -1;
 		}
 	}
@@ -1252,12 +1299,13 @@ int ggzdmod_disconnect(GGZdMod * ggzdmod)
 		/* For the ggz side, we kill the game server and close the socket */
 		
 		/* Make sure game server is dead */
-		if (ggzdmod->pid > 0)
+		if (ggzdmod->pid > 0) {
 			kill(ggzdmod->pid, SIGINT);
+			waitpid(ggzdmod->pid, NULL, 0);
+		}
 		ggzdmod->pid = -1;
 		
 		set_state(ggzdmod, GGZDMOD_STATE_DONE);
-		/* FIXME: should we wait() to get an exit status?? */
 		/* FIXME: what other cleanups should we do? */
 	} else {
 		/* For client the game side we send a game over message */
@@ -1365,7 +1413,7 @@ void _ggzdmod_handle_log(GGZdMod * ggzdmod, char *msg)
 
 
 /* Game-side event: launch event received from ggzd */
-void _ggzdmod_handle_launch_begin(GGZdMod * ggzdmod, int num_seats, int num_spectators)
+void _ggzdmod_handle_launch_begin(GGZdMod * ggzdmod, const char *game, int num_seats, int num_spectators)
 {
 #if 0
 	int bots = 0;
@@ -1383,6 +1431,7 @@ void _ggzdmod_handle_launch_begin(GGZdMod * ggzdmod, int num_seats, int num_spec
 		return;
 	}
 
+	_ggzdmod_set_game(ggzdmod, game);
 	_ggzdmod_set_num_seats(ggzdmod, num_seats);
 	_ggzdmod_set_max_num_spectators(ggzdmod, num_spectators);
 }
@@ -1470,7 +1519,8 @@ void _ggzdmod_handle_seat(GGZdMod * ggzdmod, GGZSeat *seat)
 	/* Figure out which event to use. */
 	if (seat->type == GGZ_SEAT_PLAYER
 	    && (old_seat->type == GGZ_SEAT_OPEN 
-		|| old_seat->type == GGZ_SEAT_RESERVED))
+		|| old_seat->type == GGZ_SEAT_RESERVED
+		|| old_seat->type == GGZ_SEAT_ABANDONED))
 		event = GGZDMOD_EVENT_JOIN;
 	else if (seat->type == GGZ_SEAT_OPEN
 		 && old_seat->type == GGZ_SEAT_PLAYER)
@@ -1672,6 +1722,7 @@ static GGZSpectator* spectator_copy(GGZSpectator *orig)
 
 	spectator = ggz_malloc(sizeof(GGZSpectator));
 
+	spectator->type = GGZ_SEAT_NONE;
 	spectator->num = orig->num;
 	spectator->fd = orig->fd;
 	
