@@ -4,7 +4,7 @@
  * Project: ggzdmod
  * Date: 10/14/01
  * Desc: GGZ game module functions
- * $Id: ggzdmod.h 7910 2006-03-14 13:50:41Z josef $
+ * $Id: ggzdmod.h 8945 2007-01-16 17:42:57Z josef $
  *
  * This file contains the main interface for the ggzdmod library.  This
  * library facilitates the communication between the GGZ server (ggzd)
@@ -184,7 +184,7 @@
  *     }
  * @endcode
  *
- * For more information, see the documentation at http://ggz.sf.net/.
+ * For more information, see the documentation at http://www.ggzgamingzone.org/.
  */
 
 
@@ -193,8 +193,8 @@
 
 #define GGZDMOD_VERSION_MAJOR 0
 #define GGZDMOD_VERSION_MINOR 0
-#define GGZDMOD_VERSION_MICRO 13
-#define GGZDMOD_VERSION_IFACE "5:0:1"
+#define GGZDMOD_VERSION_MICRO 14
+#define GGZDMOD_VERSION_IFACE "6:0:0"
 
 #ifdef __cplusplus
 extern "C" {
@@ -294,12 +294,6 @@ typedef enum {
 	 *  both SPECTATOR_JOIN and SPECTATOR_LEAVE. */
 	GGZDMOD_EVENT_SPECTATOR_SEAT,
 
-	/** @brief Module log request
-	 *  This event occurs when a log request happens.  This will
-	 *  only be used by the GGZ server; the game server should
-	 *  use ggzdmod_log to generate the log. */
-	GGZDMOD_EVENT_LOG,
-
 	/** @brief Data available from player
 	 *  This event occurs when there is data ready to be read from
 	 *  one of the player sockets.  The player number (an int*) is
@@ -311,18 +305,6 @@ typedef enum {
 	 *  sent some data to the game server. */
 	GGZDMOD_EVENT_SPECTATOR_DATA,
 
-	/** @brief For GGZ only.  Reports the results of a game. */
-	GGZDMOD_EVENT_GAMEREPORT,
-
-	/** @brief For GGZ only.  Reports a savegame. */
-	GGZDMOD_EVENT_SAVEGAMEREPORT,
-
-	/* GGZ-side only.  These should be moved to ggzdmod-ggz.h. */
-	GGZDMOD_EVENT_REQ_NUM_SEATS,
-	GGZDMOD_EVENT_REQ_BOOT,
-	GGZDMOD_EVENT_REQ_BOT,
-	GGZDMOD_EVENT_REQ_OPEN,
-
 	/** @brief An error has occurred
 	 *  This event occurs when a GGZdMod error has occurred.  An
 	 *  error message (a char*) will be passed as the event's data.
@@ -379,21 +361,21 @@ typedef void (*GGZdModHandler) (GGZdMod * mod, GGZdModEvent event,
  */
 typedef struct {
 	unsigned int num;	/**< Seat index; 0..(num_seats-1). */
-	GGZSeatType type;	/**< Type of seat. */
+	GGZSeatType type;	/**< Type of seat (undefined for spectators). */
 	const char *name;	/**< Name of player occupying seat. */
 	int fd;			/**< fd to communicate with seat occupant. */
 } GGZSeat;
+typedef GGZSeat GGZSpectator;
 
-/** @brief A game spectator entry
+/** @brief Is the program running in GGZ mode?
  *
- *  Spectators are handles differently from other player types.
+ *  Call this function to see if the program was actually launched by GGZ.  This can
+ *  be used to give an error message if the executable is run outside of the GGZ
+ *  environment, or for games that will run both inside and outside of GGZ.
+ *  @return A boolean value indicating whether the program is running in GGZ.
+ *  @note Should only be called by game servers, not by GGZ itself.
  */
-typedef struct {
-	unsigned int num;	/**< Spectator seat index */
-	const char *name;	/**< The spectator's name */
-	int fd;			/**< File descriptor for communication */
-} GGZSpectator;
-
+int ggzdmod_is_ggz_mode(void);
 
 /* 
  * Creation functions
